// FILE: ./assets/js/utilities/QrCodeGenerator.js
/* jshint ignore:start */
/**
 * @fileoverview
 * - Using the 'QRCode for Javascript library'
 * - Fixed dataset of 'QRCode for Javascript library' for support full-spec.
 * - this library has no dependencies.
 * 
 * @author davidshimjs
 * @see <a href="http://www.d-project.com/" target="_blank">http://www.d-project.com/</a>
 * @see <a href="http://jeromeetienne.github.com/jquery-qrcode/" target="_blank">http://jeromeetienne.github.com/jquery-qrcode/</a>
 */
var QRCode;

(function() {
    //---------------------------------------------------------------------
    // QRCode for JavaScript
    //
    // Copyright (c) 2009 Kazuhiko Arase
    //
    // URL: http://www.d-project.com/
    //
    // Licensed under the MIT license:
    //   http://www.opensource.org/licenses/mit-license.php
    //
    // The word "QR Code" is registered trademark of 
    // DENSO WAVE INCORPORATED
    //   http://www.denso-wave.com/qrcode/faqpatent-e.html
    //
    //---------------------------------------------------------------------
    function QR8bitByte(data) {
        this.mode = QRMode.MODE_8BIT_BYTE;
        this.data = data;
        this.parsedData = [];

        // Added to support UTF-8 Characters
        for (var i = 0, l = this.data.length; i < l; i++) {
            var byteArray = [];
            var code = this.data.charCodeAt(i);

            if (code > 0x10000) {
                byteArray[0] = 0xF0 | ((code & 0x1C0000) >>> 18);
                byteArray[1] = 0x80 | ((code & 0x3F000) >>> 12);
                byteArray[2] = 0x80 | ((code & 0xFC0) >>> 6);
                byteArray[3] = 0x80 | (code & 0x3F);
            } else if (code > 0x800) {
                byteArray[0] = 0xE0 | ((code & 0xF000) >>> 12);
                byteArray[1] = 0x80 | ((code & 0xFC0) >>> 6);
                byteArray[2] = 0x80 | (code & 0x3F);
            } else if (code > 0x80) {
                byteArray[0] = 0xC0 | ((code & 0x7C0) >>> 6);
                byteArray[1] = 0x80 | (code & 0x3F);
            } else {
                byteArray[0] = code;
            }

            this.parsedData.push(byteArray);
        }

        this.parsedData = Array.prototype.concat.apply([], this.parsedData);

        if (this.parsedData.length != this.data.length) {
            this.parsedData.unshift(191);
            this.parsedData.unshift(187);
            this.parsedData.unshift(239);
        }
    }

    QR8bitByte.prototype = {
        getLength: function(buffer) {
            return this.parsedData.length;
        },
        write: function(buffer) {
            for (var i = 0, l = this.parsedData.length; i < l; i++) {
                buffer.put(this.parsedData[i], 8);
            }
        }
    };

    function QRCodeModel(typeNumber, errorCorrectLevel) {
        this.typeNumber = typeNumber;
        this.errorCorrectLevel = errorCorrectLevel;
        this.modules = null;
        this.moduleCount = 0;
        this.dataCache = null;
        this.dataList = [];
    }

    QRCodeModel.prototype = {
        addData: function(data) {
            var newData = new QR8bitByte(data);
            this.dataList.push(newData);
            this.dataCache = null;
        },
        isDark: function(row, col) {
            if (row < 0 || this.moduleCount <= row || col < 0 || this.moduleCount <= col) {
                throw new Error(row + "," + col);
            }
            return this.modules[row][col];
        },
        getModuleCount: function() {
            return this.moduleCount;
        },
        make: function() {
            this.makeImpl(false, this.getBestMaskPattern());
        },
        makeImpl: function(test, maskPattern) {
            this.moduleCount = this.typeNumber * 4 + 17;
            this.modules = new Array(this.moduleCount);
            for (var row = 0; row < this.moduleCount; row++) {
                this.modules[row] = new Array(this.moduleCount);
                for (var col = 0; col < this.moduleCount; col++) {
                    this.modules[row][col] = null;
                }
            }
            this.setupPositionProbePattern(0, 0);
            this.setupPositionProbePattern(this.moduleCount - 7, 0);
            this.setupPositionProbePattern(0, this.moduleCount - 7);
            this.setupPositionAdjustPattern();
            this.setupTimingPattern();
            this.setupTypeInfo(test, maskPattern);
            if (this.typeNumber >= 7) {
                this.setupTypeNumber(test);
            }
            if (this.dataCache == null) {
                this.dataCache = QRCodeModel.createData(this.typeNumber, this.errorCorrectLevel, this.dataList);
            }
            this.mapData(this.dataCache, maskPattern);
        },
        setupPositionProbePattern: function(row, col) {
            for (var r = -1; r <= 7; r++) {
                if (row + r <= -1 || this.moduleCount <= row + r) continue;
                for (var c = -1; c <= 7; c++) {
                    if (col + c <= -1 || this.moduleCount <= col + c) continue;
                    if ((0 <= r && r <= 6 && (c == 0 || c == 6)) || (0 <= c && c <= 6 && (r == 0 || r == 6)) || (2 <= r && r <= 4 && 2 <= c && c <= 4)) {
                        this.modules[row + r][col + c] = true;
                    } else {
                        this.modules[row + r][col + c] = false;
                    }
                }
            }
        },
        getBestMaskPattern: function() {
            var minLostPoint = 0;
            var pattern = 0;
            for (var i = 0; i < 8; i++) {
                this.makeImpl(true, i);
                var lostPoint = QRUtil.getLostPoint(this);
                if (i == 0 || minLostPoint > lostPoint) {
                    minLostPoint = lostPoint;
                    pattern = i;
                }
            }
            return pattern;
        },
        createMovieClip: function(target_mc, instance_name, depth) {
            var qr_mc = target_mc.createEmptyMovieClip(instance_name, depth);
            var cs = 1;
            this.make();
            for (var row = 0; row < this.modules.length; row++) {
                var y = row * cs;
                for (var col = 0; col < this.modules[row].length; col++) {
                    var x = col * cs;
                    var dark = this.modules[row][col];
                    if (dark) {
                        qr_mc.beginFill(0, 100);
                        qr_mc.moveTo(x, y);
                        qr_mc.lineTo(x + cs, y);
                        qr_mc.lineTo(x + cs, y + cs);
                        qr_mc.lineTo(x, y + cs);
                        qr_mc.endFill();
                    }
                }
            }
            return qr_mc;
        },
        setupTimingPattern: function() {
            for (var r = 8; r < this.moduleCount - 8; r++) {
                if (this.modules[r][6] != null) {
                    continue;
                }
                this.modules[r][6] = (r % 2 == 0);
            }
            for (var c = 8; c < this.moduleCount - 8; c++) {
                if (this.modules[6][c] != null) {
                    continue;
                }
                this.modules[6][c] = (c % 2 == 0);
            }
        },
        setupPositionAdjustPattern: function() {
            var pos = QRUtil.getPatternPosition(this.typeNumber);
            for (var i = 0; i < pos.length; i++) {
                for (var j = 0; j < pos.length; j++) {
                    var row = pos[i];
                    var col = pos[j];
                    if (this.modules[row][col] != null) {
                        continue;
                    }
                    for (var r = -2; r <= 2; r++) {
                        for (var c = -2; c <= 2; c++) {
                            if (r == -2 || r == 2 || c == -2 || c == 2 || (r == 0 && c == 0)) {
                                this.modules[row + r][col + c] = true;
                            } else {
                                this.modules[row + r][col + c] = false;
                            }
                        }
                    }
                }
            }
        },
        setupTypeNumber: function(test) {
            var bits = QRUtil.getBCHTypeNumber(this.typeNumber);
            for (var i = 0; i < 18; i++) {
                var mod = (!test && ((bits >> i) & 1) == 1);
                this.modules[Math.floor(i / 3)][i % 3 + this.moduleCount - 8 - 3] = mod;
            }
            for (var i = 0; i < 18; i++) {
                var mod = (!test && ((bits >> i) & 1) == 1);
                this.modules[i % 3 + this.moduleCount - 8 - 3][Math.floor(i / 3)] = mod;
            }
        },
        setupTypeInfo: function(test, maskPattern) {
            var data = (this.errorCorrectLevel << 3) | maskPattern;
            var bits = QRUtil.getBCHTypeInfo(data);
            for (var i = 0; i < 15; i++) {
                var mod = (!test && ((bits >> i) & 1) == 1);
                if (i < 6) {
                    this.modules[i][8] = mod;
                } else if (i < 8) {
                    this.modules[i + 1][8] = mod;
                } else {
                    this.modules[this.moduleCount - 15 + i][8] = mod;
                }
            }
            for (var i = 0; i < 15; i++) {
                var mod = (!test && ((bits >> i) & 1) == 1);
                if (i < 8) {
                    this.modules[8][this.moduleCount - i - 1] = mod;
                } else if (i < 9) {
                    this.modules[8][15 - i - 1 + 1] = mod;
                } else {
                    this.modules[8][15 - i - 1] = mod;
                }
            }
            this.modules[this.moduleCount - 8][8] = (!test);
        },
        mapData: function(data, maskPattern) {
            var inc = -1;
            var row = this.moduleCount - 1;
            var bitIndex = 7;
            var byteIndex = 0;
            for (var col = this.moduleCount - 1; col > 0; col -= 2) {
                if (col == 6) col--;
                while (true) {
                    for (var c = 0; c < 2; c++) {
                        if (this.modules[row][col - c] == null) {
                            var dark = false;
                            if (byteIndex < data.length) {
                                dark = (((data[byteIndex] >>> bitIndex) & 1) == 1);
                            }
                            var mask = QRUtil.getMask(maskPattern, row, col - c);
                            if (mask) {
                                dark = !dark;
                            }
                            this.modules[row][col - c] = dark;
                            bitIndex--;
                            if (bitIndex == -1) {
                                byteIndex++;
                                bitIndex = 7;
                            }
                        }
                    }
                    row += inc;
                    if (row < 0 || this.moduleCount <= row) {
                        row -= inc;
                        inc = -inc;
                        break;
                    }
                }
            }
        }
    };
    QRCodeModel.PAD0 = 0xEC;
    QRCodeModel.PAD1 = 0x11;
    QRCodeModel.createData = function(typeNumber, errorCorrectLevel, dataList) {
        var rsBlocks = QRRSBlock.getRSBlocks(typeNumber, errorCorrectLevel);
        var buffer = new QRBitBuffer();
        for (var i = 0; i < dataList.length; i++) {
            var data = dataList[i];
            buffer.put(data.mode, 4);
            buffer.put(data.getLength(), QRUtil.getLengthInBits(data.mode, typeNumber));
            data.write(buffer);
        }
        var totalDataCount = 0;
        for (var i = 0; i < rsBlocks.length; i++) {
            totalDataCount += rsBlocks[i].dataCount;
        }
        if (buffer.getLengthInBits() > totalDataCount * 8) {
            throw new Error("code length overflow. (" +
                buffer.getLengthInBits() +
                ">" +
                totalDataCount * 8 +
                ")");
        }
        if (buffer.getLengthInBits() + 4 <= totalDataCount * 8) {
            buffer.put(0, 4);
        }
        while (buffer.getLengthInBits() % 8 != 0) {
            buffer.putBit(false);
        }
        while (true) {
            if (buffer.getLengthInBits() >= totalDataCount * 8) {
                break;
            }
            buffer.put(QRCodeModel.PAD0, 8);
            if (buffer.getLengthInBits() >= totalDataCount * 8) {
                break;
            }
            buffer.put(QRCodeModel.PAD1, 8);
        }
        return QRCodeModel.createBytes(buffer, rsBlocks);
    };
    QRCodeModel.createBytes = function(buffer, rsBlocks) {
        var offset = 0;
        var maxDcCount = 0;
        var maxEcCount = 0;
        var dcdata = new Array(rsBlocks.length);
        var ecdata = new Array(rsBlocks.length);
        for (var r = 0; r < rsBlocks.length; r++) {
            var dcCount = rsBlocks[r].dataCount;
            var ecCount = rsBlocks[r].totalCount - dcCount;
            maxDcCount = Math.max(maxDcCount, dcCount);
            maxEcCount = Math.max(maxEcCount, ecCount);
            dcdata[r] = new Array(dcCount);
            for (var i = 0; i < dcdata[r].length; i++) {
                dcdata[r][i] = 0xff & buffer.buffer[i + offset];
            }
            offset += dcCount;
            var rsPoly = QRUtil.getErrorCorrectPolynomial(ecCount);
            var rawPoly = new QRPolynomial(dcdata[r], rsPoly.getLength() - 1);
            var modPoly = rawPoly.mod(rsPoly);
            ecdata[r] = new Array(rsPoly.getLength() - 1);
            for (var i = 0; i < ecdata[r].length; i++) {
                var modIndex = i + modPoly.getLength() - ecdata[r].length;
                ecdata[r][i] = (modIndex >= 0) ? modPoly.get(modIndex) : 0;
            }
        }
        var totalCodeCount = 0;
        for (var i = 0; i < rsBlocks.length; i++) {
            totalCodeCount += rsBlocks[i].totalCount;
        }
        var data = new Array(totalCodeCount);
        var index = 0;
        for (var i = 0; i < maxDcCount; i++) {
            for (var r = 0; r < rsBlocks.length; r++) {
                if (i < dcdata[r].length) {
                    data[index++] = dcdata[r][i];
                }
            }
        }
        for (var i = 0; i < maxEcCount; i++) {
            for (var r = 0; r < rsBlocks.length; r++) {
                if (i < ecdata[r].length) {
                    data[index++] = ecdata[r][i];
                }
            }
        }
        return data;
    };
    var QRMode = {
        MODE_NUMBER: 1 << 0,
        MODE_ALPHA_NUM: 1 << 1,
        MODE_8BIT_BYTE: 1 << 2,
        MODE_KANJI: 1 << 3
    };
    var QRErrorCorrectLevel = {
        L: 1,
        M: 0,
        Q: 3,
        H: 2
    };
    var QRMaskPattern = {
        PATTERN000: 0,
        PATTERN001: 1,
        PATTERN010: 2,
        PATTERN011: 3,
        PATTERN100: 4,
        PATTERN101: 5,
        PATTERN110: 6,
        PATTERN111: 7
    };
    var QRUtil = {
        PATTERN_POSITION_TABLE: [[], [6, 18], [6, 22], [6, 26], [6, 30], [6, 34], [6, 22, 38], [6, 24, 42], [6, 26, 46], [6, 28, 50], [6, 30, 54], [6, 32, 58], [6, 34, 62], [6, 26, 46, 66], [6, 26, 48, 70], [6, 26, 50, 74], [6, 30, 54, 78], [6, 30, 56, 82], [6, 30, 58, 86], [6, 34, 62, 90], [6, 28, 50, 72, 94], [6, 26, 50, 74, 98], [6, 30, 54, 78, 102], [6, 28, 54, 80, 106], [6, 32, 58, 84, 110], [6, 30, 58, 86, 114], [6, 34, 62, 90, 118], [6, 26, 50, 74, 98, 122], [6, 30, 54, 78, 102, 126], [6, 26, 52, 78, 104, 130], [6, 30, 56, 82, 108, 134], [6, 34, 60, 86, 112, 138], [6, 30, 58, 86, 114, 142], [6, 34, 62, 90, 118, 146], [6, 30, 54, 78, 102, 126, 150], [6, 24, 50, 76, 102, 128, 154], [6, 28, 54, 80, 106, 132, 158], [6, 32, 58, 84, 110, 136, 162], [6, 26, 54, 82, 110, 138, 166], [6, 30, 58, 86, 114, 142, 170]],
        G15: (1 << 10) | (1 << 8) | (1 << 5) | (1 << 4) | (1 << 2) | (1 << 1) | (1 << 0),
        G18: (1 << 12) | (1 << 11) | (1 << 10) | (1 << 9) | (1 << 8) | (1 << 5) | (1 << 2) | (1 << 0),
        G15_MASK: (1 << 14) | (1 << 12) | (1 << 10) | (1 << 4) | (1 << 1),
        getBCHTypeInfo: function(data) {
            var d = data << 10;
            while (QRUtil.getBCHDigit(d) - QRUtil.getBCHDigit(QRUtil.G15) >= 0) {
                d ^= (QRUtil.G15 << (QRUtil.getBCHDigit(d) - QRUtil.getBCHDigit(QRUtil.G15)));
            }
            return ((data << 10) | d) ^ QRUtil.G15_MASK;
        },
        getBCHTypeNumber: function(data) {
            var d = data << 12;
            while (QRUtil.getBCHDigit(d) - QRUtil.getBCHDigit(QRUtil.G18) >= 0) {
                d ^= (QRUtil.G18 << (QRUtil.getBCHDigit(d) - QRUtil.getBCHDigit(QRUtil.G18)));
            }
            return (data << 12) | d;
        },
        getBCHDigit: function(data) {
            var digit = 0;
            while (data != 0) {
                digit++;
                data >>>= 1;
            }
            return digit;
        },
        getPatternPosition: function(typeNumber) {
            return QRUtil.PATTERN_POSITION_TABLE[typeNumber - 1];
        },
        getMask: function(maskPattern, i, j) {
            switch (maskPattern) {
                case QRMaskPattern.PATTERN000:
                    return (i + j) % 2 == 0;
                case QRMaskPattern.PATTERN001:
                    return i % 2 == 0;
                case QRMaskPattern.PATTERN010:
                    return j % 3 == 0;
                case QRMaskPattern.PATTERN011:
                    return (i + j) % 3 == 0;
                case QRMaskPattern.PATTERN100:
                    return (Math.floor(i / 2) + Math.floor(j / 3)) % 2 == 0;
                case QRMaskPattern.PATTERN101:
                    return (i * j) % 2 + (i * j) % 3 == 0;
                case QRMaskPattern.PATTERN110:
                    return ((i * j) % 2 + (i * j) % 3) % 2 == 0;
                case QRMaskPattern.PATTERN111:
                    return ((i * j) % 3 + (i + j) % 2) % 2 == 0;
                default:
                    throw new Error("bad maskPattern:" + maskPattern);
            }
        },
        getErrorCorrectPolynomial: function(errorCorrectLength) {
            var a = new QRPolynomial([1], 0);
            for (var i = 0; i < errorCorrectLength; i++) {
                a = a.multiply(new QRPolynomial([1, QRMath.gexp(i)], 0));
            }
            return a;
        },
        getLengthInBits: function(mode, type) {
            if (1 <= type && type < 10) {
                switch (mode) {
                    case QRMode.MODE_NUMBER:
                        return 10;
                    case QRMode.MODE_ALPHA_NUM:
                        return 9;
                    case QRMode.MODE_8BIT_BYTE:
                        return 8;
                    case QRMode.MODE_KANJI:
                        return 8;
                    default:
                        throw new Error("mode:" + mode);
                }
            } else if (type < 27) {
                switch (mode) {
                    case QRMode.MODE_NUMBER:
                        return 12;
                    case QRMode.MODE_ALPHA_NUM:
                        return 11;
                    case QRMode.MODE_8BIT_BYTE:
                        return 16;
                    case QRMode.MODE_KANJI:
                        return 10;
                    default:
                        throw new Error("mode:" + mode);
                }
            } else if (type < 41) {
                switch (mode) {
                    case QRMode.MODE_NUMBER:
                        return 14;
                    case QRMode.MODE_ALPHA_NUM:
                        return 13;
                    case QRMode.MODE_8BIT_BYTE:
                        return 16;
                    case QRMode.MODE_KANJI:
                        return 12;
                    default:
                        throw new Error("mode:" + mode);
                }
            } else {
                throw new Error("type:" + type);
            }
        },
        getLostPoint: function(qrCode) {
            var moduleCount = qrCode.getModuleCount();
            var lostPoint = 0;
            for (var row = 0; row < moduleCount; row++) {
                for (var col = 0; col < moduleCount; col++) {
                    var sameCount = 0;
                    var dark = qrCode.isDark(row, col);
                    for (var r = -1; r <= 1; r++) {
                        if (row + r < 0 || moduleCount <= row + r) {
                            continue;
                        }
                        for (var c = -1; c <= 1; c++) {
                            if (col + c < 0 || moduleCount <= col + c) {
                                continue;
                            }
                            if (r == 0 && c == 0) {
                                continue;
                            }
                            if (dark == qrCode.isDark(row + r, col + c)) {
                                sameCount++;
                            }
                        }
                    }
                    if (sameCount > 5) {
                        lostPoint += (3 + sameCount - 5);
                    }
                }
            }
            for (var row = 0; row < moduleCount - 1; row++) {
                for (var col = 0; col < moduleCount - 1; col++) {
                    var count = 0;
                    if (qrCode.isDark(row, col)) count++;
                    if (qrCode.isDark(row + 1, col)) count++;
                    if (qrCode.isDark(row, col + 1)) count++;
                    if (qrCode.isDark(row + 1, col + 1)) count++;
                    if (count == 0 || count == 4) {
                        lostPoint += 3;
                    }
                }
            }
            for (var row = 0; row < moduleCount; row++) {
                for (var col = 0; col < moduleCount - 6; col++) {
                    if (qrCode.isDark(row, col) && !qrCode.isDark(row, col + 1) && qrCode.isDark(row, col + 2) && qrCode.isDark(row, col + 3) && qrCode.isDark(row, col + 4) && !qrCode.isDark(row, col + 5) && qrCode.isDark(row, col + 6)) {
                        lostPoint += 40;
                    }
                }
            }
            for (var col = 0; col < moduleCount; col++) {
                for (var row = 0; row < moduleCount - 6; row++) {
                    if (qrCode.isDark(row, col) && !qrCode.isDark(row + 1, col) && qrCode.isDark(row + 2, col) && qrCode.isDark(row + 3, col) && qrCode.isDark(row + 4, col) && !qrCode.isDark(row + 5, col) && qrCode.isDark(row + 6, col)) {
                        lostPoint += 40;
                    }
                }
            }
            var darkCount = 0;
            for (var col = 0; col < moduleCount; col++) {
                for (var row = 0; row < moduleCount; row++) {
                    if (qrCode.isDark(row, col)) {
                        darkCount++;
                    }
                }
            }
            var ratio = Math.abs(100 * darkCount / moduleCount / moduleCount - 50) / 5;
            lostPoint += ratio * 10;
            return lostPoint;
        }
    };
    var QRMath = {
        glog: function(n) {
            if (n < 1) {
                throw new Error("glog(" + n + ")");
            }
            return QRMath.LOG_TABLE[n];
        },
        gexp: function(n) {
            while (n < 0) {
                n += 255;
            }
            while (n >= 256) {
                n -= 255;
            }
            return QRMath.EXP_TABLE[n];
        },
        EXP_TABLE: new Array(256),
        LOG_TABLE: new Array(256)
    };
    for (var i = 0; i < 8; i++) {
        QRMath.EXP_TABLE[i] = 1 << i;
    }
    for (var i = 8; i < 256; i++) {
        QRMath.EXP_TABLE[i] = QRMath.EXP_TABLE[i - 4] ^ QRMath.EXP_TABLE[i - 5] ^ QRMath.EXP_TABLE[i - 6] ^ QRMath.EXP_TABLE[i - 8];
    }
    for (var i = 0; i < 255; i++) {
        QRMath.LOG_TABLE[QRMath.EXP_TABLE[i]] = i;
    }

    function QRPolynomial(num, shift) {
        if (num.length == undefined) {
            throw new Error(num.length + "/" + shift);
        }
        var offset = 0;
        while (offset < num.length && num[offset] == 0) {
            offset++;
        }
        this.num = new Array(num.length - offset + shift);
        for (var i = 0; i < num.length - offset; i++) {
            this.num[i] = num[i + offset];
        }
    }
    QRPolynomial.prototype = {
        get: function(index) {
            return this.num[index];
        },
        getLength: function() {
            return this.num.length;
        },
        multiply: function(e) {
            var num = new Array(this.getLength() + e.getLength() - 1);
            for (var i = 0; i < this.getLength(); i++) {
                for (var j = 0; j < e.getLength(); j++) {
                    num[i + j] ^= QRMath.gexp(QRMath.glog(this.get(i)) + QRMath.glog(e.get(j)));
                }
            }
            return new QRPolynomial(num, 0);
        },
        mod: function(e) {
            if (this.getLength() - e.getLength() < 0) {
                return this;
            }
            var ratio = QRMath.glog(this.get(0)) - QRMath.glog(e.get(0));
            var num = new Array(this.getLength());
            for (var i = 0; i < this.getLength(); i++) {
                num[i] = this.get(i);
            }
            for (var i = 0; i < e.getLength(); i++) {
                num[i] ^= QRMath.gexp(QRMath.glog(e.get(i)) + ratio);
            }
            return new QRPolynomial(num, 0).mod(e);
        }
    };

    function QRRSBlock(totalCount, dataCount) {
        this.totalCount = totalCount;
        this.dataCount = dataCount;
    }
    QRRSBlock.RS_BLOCK_TABLE = [[1, 26, 19], [1, 26, 16], [1, 26, 13], [1, 26, 9], [1, 44, 34], [1, 44, 28], [1, 44, 22], [1, 44, 16], [1, 70, 55], [1, 70, 44], [2, 35, 17], [2, 35, 13], [1, 100, 80], [2, 50, 32], [2, 50, 24], [4, 25, 9], [1, 134, 108], [2, 67, 43], [2, 33, 15, 2, 34, 16], [2, 33, 11, 2, 34, 12], [2, 86, 68], [4, 43, 27], [4, 43, 19], [4, 43, 15], [2, 98, 78], [4, 49, 31], [2, 32, 14, 4, 33, 15], [4, 39, 13, 1, 40, 14], [2, 121, 97], [2, 60, 38, 2, 61, 39], [4, 40, 18, 2, 41, 19], [4, 40, 14, 2, 41, 15], [2, 146, 116], [3, 58, 36, 2, 59, 37], [4, 36, 16, 4, 37, 17], [4, 36, 12, 4, 37, 13], [2, 86, 68, 2, 87, 69], [4, 69, 43, 1, 70, 44], [6, 43, 19, 2, 44, 20], [6, 43, 15, 2, 44, 16], [4, 101, 81], [1, 80, 50, 4, 81, 51], [4, 50, 22, 4, 51, 23], [3, 36, 12, 8, 37, 13], [2, 116, 92, 2, 117, 93], [6, 58, 36, 2, 59, 37], [4, 46, 20, 6, 47, 21], [7, 42, 14, 4, 43, 15], [4, 133, 107], [8, 59, 37, 1, 60, 38], [8, 44, 20, 4, 45, 21], [12, 33, 11, 4, 34, 12], [3, 145, 115, 1, 146, 116], [4, 64, 40, 5, 65, 41], [11, 36, 16, 5, 37, 17], [11, 36, 12, 5, 37, 13], [5, 109, 87, 1, 110, 88], [5, 65, 41, 5, 66, 42], [5, 54, 24, 7, 55, 25], [11, 36, 12], [5, 122, 98, 1, 123, 99], [7, 73, 45, 3, 74, 46], [15, 43, 19, 2, 44, 20], [3, 45, 15, 13, 46, 16], [1, 135, 107, 5, 136, 108], [10, 74, 46, 1, 75, 47], [1, 50, 22, 15, 51, 23], [2, 42, 14, 17, 43, 15], [5, 150, 120, 1, 151, 121], [9, 69, 43, 4, 70, 44], [17, 50, 22, 1, 51, 23], [2, 42, 14, 19, 43, 15], [3, 141, 113, 4, 142, 114], [3, 70, 44, 11, 71, 45], [17, 47, 21, 4, 48, 22], [9, 39, 13, 16, 40, 14], [3, 135, 107, 5, 136, 108], [3, 67, 41, 13, 68, 42], [15, 54, 24, 5, 55, 25], [15, 43, 15, 10, 44, 16], [4, 144, 116, 4, 145, 117], [17, 68, 42], [17, 50, 22, 6, 51, 23], [19, 46, 16, 6, 47, 17], [2, 139, 111, 7, 140, 112], [17, 74, 46], [7, 54, 24, 16, 55, 25], [34, 37, 13], [4, 151, 121, 5, 152, 122], [4, 75, 47, 14, 76, 48], [11, 54, 24, 14, 55, 25], [16, 45, 15, 14, 46, 16], [6, 147, 117, 4, 148, 118], [6, 73, 45, 14, 74, 46], [11, 54, 24, 16, 55, 25], [30, 46, 16, 2, 47, 17], [8, 132, 106, 4, 133, 107], [8, 75, 47, 13, 76, 48], [7, 54, 24, 22, 55, 25], [22, 45, 15, 13, 46, 16], [10, 142, 114, 2, 143, 115], [19, 74, 46, 4, 75, 47], [28, 50, 22, 6, 51, 23], [33, 46, 16, 4, 47, 17], [8, 152, 122, 4, 153, 123], [22, 73, 45, 3, 74, 46], [8, 53, 23, 26, 54, 24], [12, 45, 15, 28, 46, 16], [3, 147, 117, 10, 148, 118], [3, 73, 45, 23, 74, 46], [4, 54, 24, 31, 55, 25], [11, 45, 15, 31, 46, 16], [7, 146, 116, 7, 147, 117], [21, 73, 45, 7, 74, 46], [1, 53, 23, 37, 54, 24], [19, 45, 15, 26, 46, 16], [5, 145, 115, 10, 146, 116], [19, 75, 47, 10, 76, 48], [15, 54, 24, 25, 55, 25], [23, 45, 15, 25, 46, 16], [13, 145, 115, 3, 146, 116], [2, 74, 46, 29, 75, 47], [42, 54, 24, 1, 55, 25], [23, 45, 15, 28, 46, 16], [17, 145, 115], [10, 74, 46, 23, 75, 47], [10, 54, 24, 35, 55, 25], [19, 45, 15, 35, 46, 16], [17, 145, 115, 1, 146, 116], [14, 74, 46, 21, 75, 47], [29, 54, 24, 19, 55, 25], [11, 45, 15, 46, 46, 16], [13, 145, 115, 6, 146, 116], [14, 74, 46, 23, 75, 47], [44, 54, 24, 7, 55, 25], [59, 46, 16, 1, 47, 17], [12, 151, 121, 7, 152, 122], [12, 75, 47, 26, 76, 48], [39, 54, 24, 14, 55, 25], [22, 45, 15, 41, 46, 16], [6, 151, 121, 14, 152, 122], [6, 75, 47, 34, 76, 48], [46, 54, 24, 10, 55, 25], [2, 45, 15, 64, 46, 16], [17, 152, 122, 4, 153, 123], [29, 74, 46, 14, 75, 47], [49, 54, 24, 10, 55, 25], [24, 45, 15, 46, 46, 16], [4, 152, 122, 18, 153, 123], [13, 74, 46, 32, 75, 47], [48, 54, 24, 14, 55, 25], [42, 45, 15, 32, 46, 16], [20, 147, 117, 4, 148, 118], [40, 75, 47, 7, 76, 48], [43, 54, 24, 22, 55, 25], [10, 45, 15, 67, 46, 16], [19, 148, 118, 6, 149, 119], [18, 75, 47, 31, 76, 48], [34, 54, 24, 34, 55, 25], [20, 45, 15, 61, 46, 16]];
    QRRSBlock.getRSBlocks = function(typeNumber, errorCorrectLevel) {
        var rsBlock = QRRSBlock.getRsBlockTable(typeNumber, errorCorrectLevel);
        if (rsBlock == undefined) {
            throw new Error("bad rs block @ typeNumber:" + typeNumber + "/errorCorrectLevel:" + errorCorrectLevel);
        }
        var length = rsBlock.length / 3;
        var list = [];
        for (var i = 0; i < length; i++) {
            var count = rsBlock[i * 3 + 0];
            var totalCount = rsBlock[i * 3 + 1];
            var dataCount = rsBlock[i * 3 + 2];
            for (var j = 0; j < count; j++) {
                list.push(new QRRSBlock(totalCount, dataCount));
            }
        }
        return list;
    };
    QRRSBlock.getRsBlockTable = function(typeNumber, errorCorrectLevel) {
        switch (errorCorrectLevel) {
            case QRErrorCorrectLevel.L:
                return QRRSBlock.RS_BLOCK_TABLE[(typeNumber - 1) * 4 + 0];
            case QRErrorCorrectLevel.M:
                return QRRSBlock.RS_BLOCK_TABLE[(typeNumber - 1) * 4 + 1];
            case QRErrorCorrectLevel.Q:
                return QRRSBlock.RS_BLOCK_TABLE[(typeNumber - 1) * 4 + 2];
            case QRErrorCorrectLevel.H:
                return QRRSBlock.RS_BLOCK_TABLE[(typeNumber - 1) * 4 + 3];
            default:
                return undefined;
        }
    };

    function QRBitBuffer() {
        this.buffer = [];
        this.length = 0;
    }
    QRBitBuffer.prototype = {
        get: function(index) {
            var bufIndex = Math.floor(index / 8);
            return ((this.buffer[bufIndex] >>> (7 - index % 8)) & 1) == 1;
        },
        put: function(num, length) {
            for (var i = 0; i < length; i++) {
                this.putBit(((num >>> (length - i - 1)) & 1) == 1);
            }
        },
        getLengthInBits: function() {
            return this.length;
        },
        putBit: function(bit) {
            var bufIndex = Math.floor(this.length / 8);
            if (this.buffer.length <= bufIndex) {
                this.buffer.push(0);
            }
            if (bit) {
                this.buffer[bufIndex] |= (0x80 >>> (this.length % 8));
            }
            this.length++;
        }
    };
    var QRCodeLimitLength = [[17, 14, 11, 7], [32, 26, 20, 14], [53, 42, 32, 24], [78, 62, 46, 34], [106, 84, 60, 44], [134, 106, 74, 58], [154, 122, 86, 64], [192, 152, 108, 84], [230, 180, 130, 98], [271, 213, 151, 119], [321, 251, 177, 137], [367, 287, 203, 155], [425, 331, 241, 177], [458, 362, 258, 194], [520, 412, 292, 220], [586, 450, 322, 250], [644, 504, 364, 280], [718, 560, 394, 310], [792, 624, 442, 338], [858, 666, 482, 382], [929, 711, 509, 403], [1003, 779, 565, 439], [1091, 857, 611, 461], [1171, 911, 661, 511], [1273, 997, 715, 535], [1367, 1059, 751, 593], [1465, 1125, 805, 625], [1528, 1190, 868, 658], [1628, 1264, 908, 698], [1732, 1370, 982, 742], [1840, 1452, 1030, 790], [1952, 1538, 1112, 842], [2068, 1628, 1168, 898], [2188, 1722, 1228, 958], [2303, 1809, 1283, 983], [2431, 1911, 1351, 1051], [2563, 1989, 1423, 1093], [2699, 2099, 1499, 1139], [2809, 2213, 1579, 1219], [2953, 2331, 1663, 1273]];

    function _isSupportCanvas() {
        return typeof CanvasRenderingContext2D != "undefined";
    }

    // android 2.x doesn't support Data-URI spec
    function _getAndroid() {
        var android = false;
        var sAgent = navigator.userAgent;

        if (/android/i.test(sAgent)) { // android
            android = true;
            var aMat = sAgent.toString().match(/android ([0-9]\.[0-9])/i);

            if (aMat && aMat[1]) {
                android = parseFloat(aMat[1]);
            }
        }

        return android;
    }

    var svgDrawer = (function() {

        var Drawing = function(el, htOption) {
            this._el = el;
            this._htOption = htOption;
        };

        Drawing.prototype.draw = function(oQRCode) {
            var _htOption = this._htOption;
            var _el = this._el;
            var nCount = oQRCode.getModuleCount();
            var nWidth = Math.floor(_htOption.width / nCount);
            var nHeight = Math.floor(_htOption.height / nCount);

            this.clear();

            function makeSVG(tag, attrs) {
                var el = document.createElementNS('http://www.w3.org/2000/svg', tag);
                for (var k in attrs)
                    if (attrs.hasOwnProperty(k)) el.setAttribute(k, attrs[k]);
                return el;
            }

            var svg = makeSVG("svg", {
                'viewBox': '0 0 ' + String(nCount) + " " + String(nCount),
                'width': '100%',
                'height': '100%',
                'fill': _htOption.colorLight
            });
            svg.setAttributeNS("http://www.w3.org/2000/xmlns/", "xmlns:xlink", "http://www.w3.org/1999/xlink");
            _el.appendChild(svg);

            svg.appendChild(makeSVG("rect", {
                "fill": _htOption.colorLight,
                "width": "100%",
                "height": "100%"
            }));
            svg.appendChild(makeSVG("rect", {
                "fill": _htOption.colorDark,
                "width": "1",
                "height": "1",
                "id": "template"
            }));

            for (var row = 0; row < nCount; row++) {
                for (var col = 0; col < nCount; col++) {
                    if (oQRCode.isDark(row, col)) {
                        var child = makeSVG("use", {
                            "x": String(col),
                            "y": String(row)
                        });
                        child.setAttributeNS("http://www.w3.org/1999/xlink", "href", "#template")
                        svg.appendChild(child);
                    }
                }
            }
        };
        Drawing.prototype.clear = function() {
            while (this._el.hasChildNodes())
                this._el.removeChild(this._el.lastChild);
        };
        return Drawing;
    })();

    var useSVG = document.documentElement.tagName.toLowerCase() === "svg";

    // Drawing in DOM by using Table tag
    var Drawing = useSVG ? svgDrawer : !_isSupportCanvas() ? (function() {
        var Drawing = function(el, htOption) {
            this._el = el;
            this._htOption = htOption;
        };

        /**
         * Draw the QRCode
         * 
         * @param {QRCode} oQRCode
         */
        Drawing.prototype.draw = function(oQRCode) {
            var _htOption = this._htOption;
            var _el = this._el;
            var nCount = oQRCode.getModuleCount();
            var nWidth = Math.floor(_htOption.width / nCount);
            var nHeight = Math.floor(_htOption.height / nCount);
            var aHTML = ['<table style="border:0;border-collapse:collapse;">'];

            for (var row = 0; row < nCount; row++) {
                aHTML.push('<tr>');

                for (var col = 0; col < nCount; col++) {
                    aHTML.push('<td style="border:0;border-collapse:collapse;padding:0;margin:0;width:' + nWidth + 'px;height:' + nHeight + 'px;background-color:' + (oQRCode.isDark(row, col) ? _htOption.colorDark : _htOption.colorLight) + ';"></td>');
                }

                aHTML.push('</tr>');
            }

            aHTML.push('</table>');
            _el.innerHTML = aHTML.join('');

            // Fix the margin values as real size.
            var elTable = _el.childNodes[0];
            var nLeftMarginTable = (_htOption.width - elTable.offsetWidth) / 2;
            var nTopMarginTable = (_htOption.height - elTable.offsetHeight) / 2;

            if (nLeftMarginTable > 0 && nTopMarginTable > 0) {
                elTable.style.margin = nTopMarginTable + "px " + nLeftMarginTable + "px";
            }
        };

        /**
         * Clear the QRCode
         */
        Drawing.prototype.clear = function() {
            this._el.innerHTML = '';
        };

        return Drawing;
    })() : (function() { // Drawing in Canvas
        function _onMakeImage() {
            this._elImage.src = this._elCanvas.toDataURL("image/png");
            setLocalVar("qrcode_uri", this._elCanvas.toDataURL("image/png"));
            this._elImage.style.display = "inline-block";
            this._elCanvas.style.display = "none";
        }

        // Android 2.1 bug workaround
        // http://code.google.com/p/android/issues/detail?id=5141
        if (this._android && this._android <= 2.1) {
            var factor = 1 / window.devicePixelRatio;
            var drawImage = CanvasRenderingContext2D.prototype.drawImage;
            CanvasRenderingContext2D.prototype.drawImage = function(image, sx, sy, sw, sh, dx, dy, dw, dh) {
                if (("nodeName" in image) && /img/i.test(image.nodeName)) {
                    for (var i = arguments.length - 1; i >= 1; i--) {
                        arguments[i] = arguments[i] * factor;
                    }
                } else if (typeof dw == "undefined") {
                    arguments[1] *= factor;
                    arguments[2] *= factor;
                    arguments[3] *= factor;
                    arguments[4] *= factor;
                }

                drawImage.apply(this, arguments);
            };
        }

        /**
         * Check whether the user's browser supports Data URI or not
         * 
         * @private
         * @param {Function} fSuccess Occurs if it supports Data URI
         * @param {Function} fFail Occurs if it doesn't support Data URI
         */
        function _safeSetDataURI(fSuccess, fFail) {
            var self = this;
            self._fFail = fFail;
            self._fSuccess = fSuccess;

            // Check it just once
            if (self._bSupportDataURI === null) {
                var el = document.createElement("img");
                var fOnError = function() {
                    self._bSupportDataURI = false;

                    if (self._fFail) {
                        self._fFail.call(self);
                    }
                };
                var fOnSuccess = function() {
                    self._bSupportDataURI = true;

                    if (self._fSuccess) {
                        self._fSuccess.call(self);
                    }
                };

                el.onabort = fOnError;
                el.onerror = fOnError;
                el.onload = fOnSuccess;
                el.src = "data:image/gif;base64,iVBORw0KGgoAAAANSUhEUgAAAAUAAAAFCAYAAACNbyblAAAAHElEQVQI12P4//8/w38GIAXDIBKE0DHxgljNBAAO9TXL0Y4OHwAAAABJRU5ErkJggg=="; // the Image contains 1px data.
                return;
            } else if (self._bSupportDataURI === true && self._fSuccess) {
                self._fSuccess.call(self);
            } else if (self._bSupportDataURI === false && self._fFail) {
                self._fFail.call(self);
            }
        };

        /**
         * Drawing QRCode by using canvas
         * 
         * @constructor
         * @param {HTMLElement} el
         * @param {Object} htOption QRCode Options 
         */
        var Drawing = function(el, htOption, dataURI) {
            this._bIsPainted = false;
            this._android = _getAndroid();

            this._htOption = htOption;
            this._elCanvas = document.createElement("canvas");
            this._elCanvas.width = htOption.width;
            this._elCanvas.height = htOption.height;
            el.appendChild(this._elCanvas);
            this._el = el;
            this._oContext = this._elCanvas.getContext("2d");
            this._bIsPainted = false;
            this._elImage = document.createElement("img");
            this._elImage.alt = "qrCodeGenerated";
            this._elImage.style.display = "none";
            this._el.appendChild(this._elImage);
            this._bSupportDataURI = null;

        };

        /**
         * Draw the QRCode
         * 
         * @param {QRCode} oQRCode 
         */
        Drawing.prototype.draw = function(oQRCode) {
            var _elImage = this._elImage;
            var _oContext = this._oContext;
            var _htOption = this._htOption;

            var nCount = oQRCode.getModuleCount();
            var nWidth = _htOption.width / nCount;
            var nHeight = _htOption.height / nCount;
            var nRoundedWidth = Math.round(nWidth);
            var nRoundedHeight = Math.round(nHeight);

            _elImage.style.display = "none";
            this.clear();

            for (var row = 0; row < nCount; row++) {
                for (var col = 0; col < nCount; col++) {
                    var bIsDark = oQRCode.isDark(row, col);
                    var nLeft = col * nWidth;
                    var nTop = row * nHeight;
                    _oContext.strokeStyle = bIsDark ? _htOption.colorDark : _htOption.colorLight;
                    _oContext.lineWidth = 1;
                    _oContext.fillStyle = bIsDark ? _htOption.colorDark : _htOption.colorLight;
                    _oContext.fillRect(nLeft, nTop, nWidth, nHeight);

                    // 안티 앨리어싱 방지 처리
                    _oContext.strokeRect(
                        Math.floor(nLeft) + 0.5,
                        Math.floor(nTop) + 0.5,
                        nRoundedWidth,
                        nRoundedHeight
                    );

                    _oContext.strokeRect(
                        Math.ceil(nLeft) - 0.5,
                        Math.ceil(nTop) - 0.5,
                        nRoundedWidth,
                        nRoundedHeight
                    );
                }
            }

            this._bIsPainted = true;
        };

        /**
         * Make the image from Canvas if the browser supports Data URI.
         */
        Drawing.prototype.makeImage = function() {
            if (this._bIsPainted) {
                _safeSetDataURI.call(this, _onMakeImage);
            }
        };

        /**
         * Return whether the QRCode is painted or not
         * 
         * @return {Boolean}
         */
        Drawing.prototype.isPainted = function() {
            return this._bIsPainted;
        };

        /**
         * Clear the QRCode
         */
        Drawing.prototype.clear = function() {
            this._oContext.clearRect(0, 0, this._elCanvas.width, this._elCanvas.height);
            this._bIsPainted = false;
        };

        /**
         * @private
         * @param {Number} nNumber
         */
        Drawing.prototype.round = function(nNumber) {
            if (!nNumber) {
                return nNumber;
            }

            return Math.floor(nNumber * 1000) / 1000;
        };

        return Drawing;
    })();

    /**
     * Get the type by string length
     * 
     * @private
     * @param {String} sText
     * @param {Number} nCorrectLevel
     * @return {Number} type
     */
    function _getTypeNumber(sText, nCorrectLevel) {
        var nType = 1;
        var length = _getUTF8Length(sText);

        for (var i = 0, len = QRCodeLimitLength.length; i <= len; i++) {
            var nLimit = 0;

            switch (nCorrectLevel) {
                case QRErrorCorrectLevel.L:
                    nLimit = QRCodeLimitLength[i][0];
                    break;
                case QRErrorCorrectLevel.M:
                    nLimit = QRCodeLimitLength[i][1];
                    break;
                case QRErrorCorrectLevel.Q:
                    nLimit = QRCodeLimitLength[i][2];
                    break;
                case QRErrorCorrectLevel.H:
                    nLimit = QRCodeLimitLength[i][3];
                    break;
            }

            if (length <= nLimit) {
                break;
            } else {
                nType++;
            }
        }

        if (nType > QRCodeLimitLength.length) {
            throw new Error("Too long data");
        }

        return nType;
    }

    function _getUTF8Length(sText) {
        var replacedText = encodeURI(sText).toString().replace(/\%[0-9a-fA-F]{2}/g, 'a');
        return replacedText.length + (replacedText.length != sText ? 3 : 0);
    }

    /**
     * @class QRCode
     * @constructor
     * @example 
     * new QRCode(document.getElementById("test"), "http://jindo.dev.naver.com/collie");
     *
     * @example
     * var oQRCode = new QRCode("test", {
     *    text : "http://naver.com",
     *    width : 128,
     *    height : 128
     * });
     * 
     * oQRCode.clear(); // Clear the QRCode.
     * oQRCode.makeCode("http://map.naver.com"); // Re-create the QRCode.
     *
     * @param {HTMLElement|String} el target element or 'id' attribute of element.
     * @param {Object|String} vOption
     * @param {String} vOption.text QRCode link data
     * @param {Number} [vOption.width=256]
     * @param {Number} [vOption.height=256]
     * @param {String} [vOption.colorDark="#000000"]
     * @param {String} [vOption.colorLight="#ffffff"]
     * @param {QRCode.CorrectLevel} [vOption.correctLevel=QRCode.CorrectLevel.H] [L|M|Q|H] 
     */
    QRCode = function(el, vOption) {
        this._htOption = {
            width: 256,
            height: 256,
            typeNumber: 4,
            colorDark: "#000000",
            colorLight: "#ffffff",
            correctLevel: QRErrorCorrectLevel.H
        };

        if (typeof vOption === 'string') {
            vOption = {
                text: vOption
            };
        }

        // Overwrites options
        if (vOption) {
            for (var i in vOption) {
                this._htOption[i] = vOption[i];
            }
        }

        if (typeof el == "string") {
            el = document.getElementById(el);
        }

        if (this._htOption.useSVG) {
            Drawing = svgDrawer;
        }

        this._android = _getAndroid();
        this._el = el;
        this._oQRCode = null;
        this._oDrawing = new Drawing(this._el, this._htOption);

        if (this._htOption.text) {
            this.makeCode(this._htOption.text);
        }
    };

    /**
     * Make the QRCode
     * 
     * @param {String} sText link data
     */
    QRCode.prototype.makeCode = function(sText) {
        this._oQRCode = new QRCodeModel(_getTypeNumber(sText, this._htOption.correctLevel), this._htOption.correctLevel);
        this._oQRCode.addData(sText);
        this._oQRCode.make();
        this._oDrawing.draw(this._oQRCode);
        this.makeImage();
    };

    /**
     * Make the Image from Canvas element
     * - It occurs automatically
     * - Android below 3 doesn't support Data-URI spec.
     * 
     * @private
     */
    QRCode.prototype.makeImage = function() {
        if (typeof this._oDrawing.makeImage == "function" && (!this._android || this._android >= 3)) {
            this._oDrawing.makeImage();
        }
    };

    /**
     * Clear the QRCode
     */
    QRCode.prototype.clear = function() {
        this._oDrawing.clear();
    };

    /**
     * @name QRCode.CorrectLevel
     */
    QRCode.CorrectLevel = QRErrorCorrectLevel;
})();
/* jshint ignore:end */


// FILE: ./assets/js/utilities/Utilities.js
function Utilities(ScreenService, MessageService, General, ZhPreferences, Query, validateEngine, ZhFilterPreferences, templateManager) {

/*global Util, Configuration, $*/

    this.changeTagRichText = function(fieldValue) {
        if (typeof(fieldValue) === 'string') {
            var newFieldValue;

            if (fieldValue.substr(0, 3) == '<p>') {
                newFieldValue = fieldValue.replace(/<p>/g, '').replace(/<\/p>/g, '<br>');
            }
            else if (fieldValue.substr(0, 3) == '<p ') {
                newFieldValue = fieldValue.replace(/<p/g, '<span').replace(/<\/p>/g, '</span><br>');
            }
            else {
                newFieldValue = fieldValue;
            }

            if (fieldValue.substr(-4) == '</p>') {
                newFieldValue = newFieldValue.slice('', -4);
            }

            return newFieldValue;
        }
        else {
            return fieldValue;
        }
    };

    this.startFilter = function(widget, currentRow) {
        return ZhPreferences.getDefaultWidgetPreferences(widget.id).then(function(result) {
            if (result && result.DEFAULT_FILTER) {
                ZhPreferences.getPreferences('filter', Query.build().where('WIDGET_ID').equals(widget.id)).then(function(filters) {
                    var defaultFilter = filters.filter(function(filter) {
                        return filter.ID == result.DEFAULT_FILTER;
                    })[0];
                    var filter = {
                        ID: defaultFilter.LABEL_TO_PREFERENCE || 'EDITED',
                        FILTER_ROW: JSON.parse(defaultFilter.FILTER_ROW)
                    };

                    currentRow = filter.FILTER_ROW;

                    ZhFilterPreferences.applyFilter(widget, filter).then(function() {
                        widget.appliedFilter = widget.appliedFilter || filter;
                        widget.appliedFilter.ID = result.DEFAULT_FILTER;
                        if (defaultFilter.LABEL_TO_PREFERENCE) {
                            widget._filterInfo[0].value = defaultFilter.LABEL_TO_PREFERENCE;
                        }
                    });
                });
            }
            else {
                if (Util.isUndefinedOrNull(currentRow) || Util.isEmptyObject(currentRow)) {
                    currentRow = buildFilterCurrentRow(widget);
                }
            }
            ZhFilterPreferences.openFilterWidget(widget, currentRow);
        });
    };

    var buildFilterCurrentRow = function(widget) {
        return getFilterRowWithDefaultValues(widget.fields);
    };

    var getFilterRowWithDefaultValues = function(filterFields) {
        let filterRow = {};
        filterFields.forEach(field => {
            let defaultValue = Util.getNestedProperty(field, ["filterProperties", "defaultValue"]) || field.defaultValue;
            if (!Util.isUndefinedOrNull(defaultValue)) {
                filterRow[field.name] = Util.evaluate(defaultValue);
            }
        });
        return filterRow;
    };
    
    function formatStringToFloat(word, simbolThousands, simbolDecimals){
        word = word.split(simbolThousands).join('');
        word = word.replace(simbolDecimals, '.');    
        return word;
    }


    function verifyRange(value, validationAttrs) {
        const maxValue = validationAttrs.max || '9999999.99';
        const minValue = validationAttrs.min || 0;
        const simbolThousands = validationAttrs.thousands || '.';
        const simbolDecimals = validationAttrs.decimals || ',';
        let valid = true;
        let msg = validationAttrs.message || (MessageService.getMessageByCode("sup#NAO_ESTA_INTERVALO") + minValue + " a " + maxValue);
        
        if (typeof(value) == 'string') {
            value = formatStringToFloat(value, simbolThousands, simbolDecimals);
        }

        if (value && (parseFloat(value) > parseFloat(maxValue) || parseFloat(value) < parseFloat(minValue))) {
            valid = false;
        }
        
        return  {"valid": valid, "message": msg};
    }
    
    function validaProduto(value, validationAttrs) {

        var filtro = templateManager.container.getActiveWidget();
        var retorno = {
            "valid": true
        };
        let field = validationAttrs.field;
        let fieldProduto;
        const final = filtro.currentRow.CDPRODFIN;
        const inicial = filtro.currentRow.CDPRODINI;

        if (value) {
            fieldProduto = field == "NMPRODFIN" ? inicial : final;
            if (fieldProduto) {
                if (inicial > final) {
                    return {
                        "valid": false,
                        "message": validationAttrs.message || MessageService.getMessageByCode("sup#PRODUTO_INICIAL_MENOR_IGUAL_FINAL")
                    };
                }
            }
        }
        else if (field == "NMPRODFIN" && filtro.currentRow.CDPRODINI || field == "NMPRODINI" && filtro.currentRow.CDPRODFIN) {
            let msg = field == "NMPRODFIN" ? 'Produto final obrigatório' : 'Produto inicial obrigatório';
            return {
                "valid": false,
                "message": validationAttrs.message || ScreenService.i18n(msg)
            };
        }

        return retorno;
    }
    
    var registeredValidations = validateEngine.getMappedValidations();
    
    if (!registeredValidations.hasOwnProperty('verifyRange')) {
        validateEngine.registerValidation('verifyRange', verifyRange);
    }
    
    
    if (!registeredValidations.hasOwnProperty('validaProduto')) {
        validateEngine.registerValidation('validaProduto', validaProduto);
    }
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Utilities', Utilities);
});


// FILE: ./assets/js/services/aprovacao_alcadasService.js
function aprovacao_alcadasService(restEngine) {

    this.aprovaAlcada = function(currentRow, operacao, justificativa) {
        var row = {
            'OPERACAO': operacao,
            'NRAUTOFORNE': currentRow.NRAUTOFORNE,
            'CDFILDEST': currentRow.CDFILIAL,
            'CDFILLANC': currentRow.CDFILLANC,
            'APROVADOR': currentRow.APROVADOR,
            'VRAPROVACAO': currentRow.VRAPROVACAO,
            'NRAPROVALCADA': currentRow.NRAPROVALCADA,
            'NRCONFIGALCADA': currentRow.NRCONFIGALCADA,
            'CDOPERLOG': getLocalVar('CDOPERADOR'),
            'NRORG': getLocalVar('NRORG'),
            'JUSTIFICATIVA': justificativa
        };
        return restEngine.post('wkf#/aprovacaoAlcadas', {
            "requestType": "Row",
            "row": row
        });
    };

    this.getProgressBarData = function(currentRow) {
        return restEngine.post('wkf#/getProgressBarData', {
            "requestType": "Row",
            "row": {
                'CDFILDEST': currentRow.CDFILDEST,
                'CDFILLANC': getLocalVar('CDFILIAL'),
                'NRDEMANDA': currentRow.NRDEMANDA,
                'IDTPDEMANDA': 'AF'
            }
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('aprovacao_alcadasService', aprovacao_alcadasService);
});

// FILE: ./assets/js/services/configuracao_AlcadasService.js
function configuracao_AlcadasService(restEngine) {

    this.requestRow = function(currentRow, route) {
        return restEngine.post(
            'wkf#/' + route, 
            {
                'requestType': 'Row',
                'row': currentRow
            }
        );
    };
    
    this.saveFilialAuto = function(currentRow) {
        return restEngine.post(
            'wkf#/filial_alcada/cadastraFilialAuto',
            {
                'requestType': 'Row',
                'row': currentRow
            }
        );
    };
    
    this.atualizarAprovador = function(params) {
        return restEngine.post(
            'wkf#/atualizarAprovador',
            {
                'requestType': 'Row',
                'row': params
            }
        );
    };
    
    this.getInitializingParameter = function(filter) {
        return restEngine.post(
            'wkf#/crud/parametrizacao',
            {
                'requestType': 'FilterData',
                'filter': filter
            }
        );
    };
    
    this.retornaDnsApp = function(params) {
        return restEngine.post(
            'wkf#/crud/appconfig',
            {
                'requestType': 'FilterData',
                'filter': params
            }
        );
    };
    
    this.enviaEmailQrcode = function(params) {
        return restEngine.post(
            'wkf#/enviaEmailQrcode',
            {
                'requestType': 'Row',
                'row': params
            }
        );
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('configuracao_AlcadasService', configuracao_AlcadasService);
});

// FILE: ./assets/js/controllers/acompanhamento_aprovacao.js
function acompanhamento_aprovacao(ScreenService, templateManager, General, $timeout, Utilities) {
    var timelines, eventsMinDistance, self = this;

    this.load = function() {
        let templateUrl = {
            package: "teste",
            baseUrl: "./assets/templates/html/acompanhamento.html"
        };
        window.templateUrls.push(templateUrl);

        templateUrl = {
            package: "textTemplate",
            baseUrl: "./assets/templates/html/textTemplate.html"
        };
        window.templateUrls.push(templateUrl);
    };

    this.cancelar = function(widget) {
        ScreenService.closePopup();
        widget.activate();
    };
    
    this.checkFilterBeforeConfirm = function(args) {
        const filterRow = args.e.preference.FILTER_ROW;
        
        const widgetAlcadas = templateManager.container.getWidget('ALCADAS');
        
        let P_TDTPROGENTAF = '';
        let P_TDTINCLUSAO = '';
        
        
        if(filterRow.DTINCLUSAO.start == '' && filterRow.DTINCLUSAO.end == ''){
            P_TDTINCLUSAO = 'T';
        }
        if(filterRow.DTPROGENTAF.start == '' && filterRow.DTPROGENTAF.end == ''){
            P_TDTPROGENTAF = 'T';
        }
        
        widgetAlcadas.dataSourceFilter = [
            {
                "name": "CDFILDEST",
                "operator": "=",
                "value":filterRow.CDFILDEST && filterRow.CDFILDEST.length  ? filterRow.CDFILDEST.join('_') : 'T'
            },
            {
                "name": "NMALCADA",
                "operator": "=",
                "value": filterRow.NMALCADA && filterRow.NMALCADA.length  ? filterRow.NMALCADA.join('_') : 'T'
            },
            {
                "name": "IDCLASSIFIC",
                "operator": "LIKE",
                "value":filterRow.IDCLASSIFIC && filterRow.IDCLASSIFIC.length  ? filterRow.IDCLASSIFIC.join('_') : 'T'
            },
            {
                "name": "IDSITAFALCADA",
                "operator": "=",
                "value":filterRow.IDSITAFALCADA && filterRow.IDSITAFALCADA.length  ? filterRow.IDSITAFALCADA.join('_') : 'T'
            },
            {
                "name": "DTINCLUSAOINI",
                "operator": "=",
                "value":filterRow.DTINCLUSAO.start != '' ? filterRow.DTINCLUSAO.start : 'T'
            },
            {
                "name": "DTINCLUSAOFIN",
                "operator": "=",
                "value":filterRow.DTINCLUSAO.end != '' ? filterRow.DTINCLUSAO.end : 'T'
            },
            {
                "name": "DTPROGENTAFFIN",
                "operator": "=",
                "value": filterRow.DTPROGENTAF.end != '' ? filterRow.DTPROGENTAF.end : 'T'
            },
            {
                "name": "DTPROGENTAFINI",
                "operator": "=",
                "value": filterRow.DTPROGENTAF.start != '' ? filterRow.DTPROGENTAF.start : 'T'
            },
            {
                "name": "P_TDTPROGENTAF",
                "operator": "=",
                "value":P_TDTPROGENTAF
            },
            {
                "name": "P_TDTINCLUSAO",
                "operator": "=",
                "value": P_TDTINCLUSAO
            }
        ];
        
        if(widgetAlcadas.getField('DTPROGENTAF2')){
            widgetAlcadas.getField('DTPROGENTAF2').name = 'DTPROGENTAF';
        }
        //widgetAlcadas.dataSourceFilter = widgetAlcadas.dataSourceFilter.slice(0,10);
        

    };
    
    this.openFilter = function(widget){
        let CDFILDEST = getLocalVar('CDFILIAL');
        let NMFILIAL = getLocalVar('NMFILIAL');
        let filterRow = {CDFILDEST: [CDFILDEST], NMFILIAL: [NMFILIAL]};
	    let FILTER = [{name: 'CDFILIAL', value: [CDFILDEST], operator: 'IN'},
	    {name:"CDOPERADOR", operator: '=', value: "OPERADOR_LOGADO"}];
	    const FIELD_UNIT = widget.getField('CDFILDEST');
	    const DATASOURCE_UNIT = FIELD_UNIT.dataSource;
	    DATASOURCE_UNIT.load(FILTER).then( _ => {
            Utilities.startFilter(widget, filterRow);
	    });
    };

    this.checkFilterAfterApply  = function(args) {
        const widget = templateManager.container.getWidget('ALCADAS');
        widget.reload();
    };
    
    this.setFilterWidgetItemInconsistente = function() {
        const itemInconsistente = templateManager.container.getWidget('ITEM_INCONSISTENTE');
        
        itemInconsistente.dataSourceFilter = [
            {
                "name": "CDFILDEST",
                "operator": "=",
                "value":itemInconsistente.getParent().getParent().currentRow.CDFILDEST
            },
            {
                "name": "CDFILLANC",
                "operator": "=",
                "value":itemInconsistente.getParent().getParent().currentRow.CDFILLANC
            },
            {
                "name": "NRAUTOFORN",
                "operator": "=",
                "value":itemInconsistente.getParent().getParent().currentRow.NRDEMANDA
            }
        ];
        
        itemInconsistente.reload();
    };

    this.abrirDetalhes = function(widget) {
        const alcadasWidget = templateManager.container.getWidget("ALCADAS");
        const currentRowAlcadaWidget = alcadasWidget.currentRow;
        
        templateManager.container.getWidget("ACOMPANHAMENTO").getField("TIMELINE").dataSourceFilter = [
            {
                "name": "NRCONFIGALCADA",
                "operator": "=",
                "value": currentRowAlcadaWidget.NRCONFIGALCADA
            },
            {
                "name": "CDFILDEST",
                "operator": "=",
                "value": currentRowAlcadaWidget.CDFILDEST
            },
            {
                "name": "NRDEMANDA",
                "operator": "=",
                "value": currentRowAlcadaWidget.NRDEMANDA
            },
            {
                "name": "CDFILLANC",
                "operator": "=",
                "value": currentRowAlcadaWidget.CDFILLANC
            }
        ];

        this.loadSwipe(templateManager.container.getWidget("ACOMPANHAMENTO"));
        templateManager.container.getWidget("ACOMPANHAMENTO").getField("TIMELINE").reload();
        ScreenService.openSwipe(alcadasWidget.container.getWidget("MAIN_TABBED"));
    };

    this.loadSwipe = function(owner) {
        General.callRestEngine("wkf#/timeline", "FilterData", owner.getField("TIMELINE").dataSourceFilter).then(function(response) {
            owner.getField("TIMELINE").dataSource.data = response.dataset.timeline;
            modificarField(owner);
        }.bind(this));
    };

    this.voltar = function() {
        ScreenService.closeSwipe();
        templateManager.container.getWidget('ALCADAS').activate();

        setTimeout(function() {
            const widgetAcompanhamento = templateManager.container.getWidget("ACOMPANHAMENTO");
            
            const arrayFields = [
                "IDAPROVADO",
                "NMOPERADOR",
                "DTAPROVACAO",
                "DTINCLUSAO",
                "VRAPROVCMP",
                "APROVPRINC",
                "DSJUSTIFICA"
            ];
            
            for(let field of arrayFields) {
                widgetAcompanhamento.getField(field).value('');
            }
        }.bind(this), 100);

    };

    function modificarField(widget) {
        const timelines = $('.cd-horizontal-timeline');
        eventsMinDistance = 60;

        if (document.getElementsByClassName("cd-horizontal-timeline").length > 0) {
            if (timelines.length > 0) {
                initTimeline(timelines);
            }
        } else {
            setTimeout(function() {
                modificarField(widget);
            }.bind(this), 100);
        }
    }

    function initTimeline(timelines) {
        timelines.each(function() {
            setTimeout(function() {
                let timeline = $(this),
                    timelineComponents = {};
                //cache timeline components 
                timelineComponents.timelineWrapper = timeline.find('.events-wrapper');
                timelineComponents.eventsWrapper = timelineComponents.timelineWrapper.children('.events');
                timelineComponents.fillingLine = timelineComponents.eventsWrapper.children('.filling-line');
                timelineComponents.timelineEvents = timelineComponents.eventsWrapper.find('.content-box');
                timelineComponents.timelineNavigation = timeline.find('.cd-timeline-navigation');

                let data = templateManager.container.getWidget("ACOMPANHAMENTO").getField("TIMELINE").dataSource.data,
                    situacao = "exclamation";

                //assign a width to the timeline
                let timelineTotWidth = setTimelineWidth(timelineComponents, eventsMinDistance);
                //the timeline has been initialize - show it
                timeline.addClass('loaded');

                for (let i = 0; i < data.length; i++) {
                    if ((data[i].SITUACAO == "exclamation") || (data[i].SITUACAO == "error")) {
                        timelineComponents.timelineEvents.eq(i).addClass('selected');
                        situacao = data[i].SITUACAO;
                    }
                }

                updateOlderEvents($(".selected"));
                updateFilling($(".selected"), timelineComponents.fillingLine, timelineTotWidth, situacao);
                translateTimeline(timelineComponents, timelineTotWidth, 0);

                if (data.length > 0) {
                    const widgetAcompanhamento = templateManager.container.getWidget("ACOMPANHAMENTO");
                    
                    widgetAcompanhamento.getField("IDAPROVADO").value(formataDetalhes(data[0].IDAPROVADO));
                    widgetAcompanhamento.getField("NRDEMANDA").value(formataDetalhes(data[0].NRDEMANDA));
                    widgetAcompanhamento.getField("NMOPERADOR").value(formataDetalhes(data[0].NMOPERADOR));

                    formataDataAprovacao(templateManager.container.getWidget("ACOMPANHAMENTO").getField("DTAPROVACAO"), data[0].IDAPROVADO, data[0].DTAPROVACAO);
                    widgetAcompanhamento.getField("DTINCLUSAO").value(formataDetalhes(data[0].DTINCLUSAO));
                    
                    //widgetAcompanhamento.getField("DTPROGENTAF").value(formataDetalhes(data[0].DTPROGENTAF));
                    
                    widgetAcompanhamento.getField("VRAPROVCMP").value(formataDetalhes(data[0].VRAPROVCMP));
                    widgetAcompanhamento.getField("APROVPRINC").value(formataDetalhes(data[0].APROVPRINC));
                    widgetAcompanhamento.getField("DSJUSTIFICA").value(formataDetalhes(data[0].DSJUSTIFICA));
                    widgetAcompanhamento.getField("DSMOTVAFALCADA").value(formataDetalhes(templateManager.container.getWidget("ALCADAS").currentRow.DSMOTVAFALCADA));
                }
                
                //detect click on the next arrow
                timelineComponents.timelineNavigation.on('click', '.next', function(event) {
                    event.preventDefault();
                    updateSlide(timelineComponents, timelineTotWidth, 'next');
                });
                
                //detect click on the prev arrow
                timelineComponents.timelineNavigation.on('click', '.prev', function(event) {
                    event.preventDefault();
                    updateSlide(timelineComponents, timelineTotWidth, 'prev');
                });
                
                //detect click on the a single event - show new event content
                timelineComponents.eventsWrapper.on('click', 'a', function(event) {
                    event.preventDefault();

                    var data = templateManager.container.getWidget("ACOMPANHAMENTO").getField("TIMELINE").dataSource.data;

                    for (var i = 0; i < data.length; i++) {
                        if (data[i].NMOPERADOR == $(this).text()) {
                            const widgetAcompanhamento = templateManager.container.getWidget("ACOMPANHAMENTO");
                            
                            widgetAcompanhamento.getField("IDAPROVADO").value(formataDetalhes(data[i].IDAPROVADO));
                            widgetAcompanhamento.getField("NMOPERADOR").value(formataDetalhes(data[i].NMOPERADOR));
                            formataDataAprovacao(widgetAcompanhamento.getField("DTAPROVACAO"), data[i].IDAPROVADO, data[i].DTAPROVACAO);
                            widgetAcompanhamento.getField("DTINCLUSAO").value(formataDetalhes(data[i].DTINCLUSAO));
                            
                            //widgetAcompanhamento.getField("DTPROGENTAF").value(formataDetalhes(data[i].DTPROGENTAF));
                            
                            widgetAcompanhamento.getField("VRAPROVCMP").value(formataDetalhes(data[i].VRAPROVCMP));
                            widgetAcompanhamento.getField("APROVPRINC").value(formataDetalhes(data[i].APROVPRINC));
                            widgetAcompanhamento.getField("DSJUSTIFICA").value(formataDetalhes(data[i].DSJUSTIFICA));
                        }
                    }
                });

            }.bind(this), 200);

        });
    }
    
    function formataDataAprovacao(widget, situacao, value) {
        if (situacao == "RECUSADO") {
            widget.label = "Data de Reprovação";
        } else {
            widget.label = "Data de Aprovação";
        }


        widget.value(formataDetalhes(value));
    }

    function formataDetalhes(param) {
        if (param != null) {
            return param;
        }

        return "-";
    }

    function updateSlide(timelineComponents, timelineTotWidth, string) {
        //retrieve translateX value of timelineComponents.eventsWrapper
        const translateValue = getTranslateValue(timelineComponents.eventsWrapper),
            wrapperWidth = Number(timelineComponents.timelineWrapper.css('width').replace('px', ''));
        //translate the timeline to the left('next')/right('prev') 
        if (string == 'next') {
            translateTimeline(timelineComponents, translateValue - wrapperWidth + eventsMinDistance, wrapperWidth - timelineTotWidth);
        } else {
            translateTimeline(timelineComponents, translateValue + wrapperWidth - eventsMinDistance);
        }
    }

    function translateTimeline(timelineComponents, value, totWidth) {
        const eventsWrapper = timelineComponents.eventsWrapper.get(0);
        value = (value > 0) ? 0 : value; //only negative translate value
        value = ((typeof totWidth !== 'undefined') && value < totWidth) ? totWidth : value; //do not translate more than timeline width
        setTransformValue(eventsWrapper, 'translateX', value + 'px');
        //update navigation arrows visibility
        if (value == 0) {
            timelineComponents.timelineNavigation.find('.prev').addClass('inactive');
        } else { 
            timelineComponents.timelineNavigation.find('.prev').removeClass('inactive');
        }
        
        if (value == totWidth) {
            timelineComponents.timelineNavigation.find('.next').addClass('inactive');
        } else {
            timelineComponents.timelineNavigation.find('.next').removeClass('inactive');
        }
    }

    function updateFilling(selectedEvent, filling, totWidth, situacao) {
        //change .filling-line length according to the selected event
        const eventStyle = window.getComputedStyle(selectedEvent.get(0), null);
        
        if (eventStyle != null) {
            let eventLeft = eventStyle.getPropertyValue("left");
            const eventWidth = eventStyle.getPropertyValue("width");

            eventLeft = Number(eventLeft.replace('px', '')) + Number(eventWidth.replace('px', '')) / 2;
            
            const scaleValue = eventLeft / totWidth;
            setTransformValue(filling.get(0), 'scaleX', scaleValue, situacao);
        }
    }
    
    function setTimelineWidth(timelineComponents, width) {
        let i = 0;
        let offset = 0;
        
        const numItens = timelineComponents.timelineEvents.length;
        const widthCircles = timelineComponents.timelineNavigation.find('.next').width();
        const minSize = timelineComponents.timelineNavigation.find('.next').position().left - widthCircles;
        const totalWidth = numItens * 200;
        const widthContentBox = 150;

        if (totalWidth > minSize) {
            offset = 20;
            for (i = 0; i < numItens; i++) {
                timelineComponents.timelineEvents.eq(i).css('left', offset + 'px');
                offset += 200;
            }
        } else {
            var distance = ((minSize - (widthContentBox * numItens)) / (numItens + 1));

            offset = distance;
            for (i = 0; i < numItens; i++) {
                timelineComponents.timelineEvents.eq(i).css('left', offset + 'px');
                offset += distance + widthContentBox;
            }
        }

        if (totalWidth > minSize) {
            timelineComponents.eventsWrapper.css('width', totalWidth + 'px');
            return totalWidth;
        } else {
            timelineComponents.eventsWrapper.css('width', minSize + 'px');
            timelineComponents.timelineNavigation.find('.prev').addClass('inactive');
            timelineComponents.timelineNavigation.find('.next').addClass('inactive');
            return minSize;
        }
    }

    function updateOlderEvents(event) {
        event.parent('li').prevAll('li').children('a').addClass('older-event').end().end().nextAll('li').children('a').removeClass('older-event');
    }

    function getTranslateValue(timeline) {
        let timelineStyle = window.getComputedStyle(timeline.get(0), null),
            timelineTranslate = timelineStyle.getPropertyValue("-webkit-transform") ||
            timelineStyle.getPropertyValue("-moz-transform") ||
            timelineStyle.getPropertyValue("-ms-transform") ||
            timelineStyle.getPropertyValue("-o-transform") ||
            timelineStyle.getPropertyValue("transform");

        let translateValue;

        if (timelineTranslate.indexOf('(') >= 0) {
            timelineTranslate = timelineTranslate.split('(')[1];
            timelineTranslate = timelineTranslate.split(')')[0];
            timelineTranslate = timelineTranslate.split(',');
            translateValue = timelineTranslate[4];
        } else {
            translateValue = 0;
        }

        return Number(translateValue);
    }

    function setTransformValue(element, property, value, situacao) {

        if (situacao == "exclamation") {
            // gradient from green to yellow
            element.style.background = "linear-gradient(270deg,#E2B900,#179021 30%)";
        } else if (situacao == "error") {
            // gradient from green to red
            element.style.background = "linear-gradient(270deg,#E70000,#179021 30%)";
        } else {
            // all gray
            element.style.background = "linear-gradient(270deg,#6A6A6A,#6A6A6A 30%)";
        }

        element.style["-webkit-transform"] = property + "(" + value + ")";
        element.style["-moz-transform"] = property + "(" + value + ")";
        element.style["-ms-transform"] = property + "(" + value + ")";
        element.style["-o-transform"] = property + "(" + value + ")";
        element.style.transform = property + "(" + value + ")";
    }

    //based on http://stackoverflow.com/questions/542938/how-do-i-get-the-number-of-days-between-two-dates-in-javascript
    function parseDate(events) {
        const dateArrays = [];
        
        events.each(function() {
            let dateComp = $(this).data('date').split('/'),
                newDate = new Date(dateComp[2], dateComp[1] - 1, dateComp[0]);
            dateArrays.push(newDate);
        });
        
        return dateArrays;
    }

    function daydiff(first, second) {
        return Math.round((second - first));
    }

    function minLapse(dates) {
        //determine the minimum distance among events
        const  dateDistances = [];
        
        for (let i = 1; i < dates.length; i++) {
            let distance = daydiff(dates[i - 1], dates[i]);
            dateDistances.push(distance);
        }
        
        return Math.min.apply(null, dateDistances);
    }

    /*
    	How to tell if a DOM element is visible in the current viewport?
    	http://stackoverflow.com/questions/123999/how-to-tell-if-a-dom-element-is-visible-in-the-current-viewport
    */
    function elementInViewport(el) {
        let top = el.offsetTop;
        let left = el.offsetLeft;
        const width = el.offsetWidth;
        const height = el.offsetHeight;

        while (el.offsetParent) {
            el = el.offsetParent;
            top += el.offsetTop;
            left += el.offsetLeft;
        }

        return (
            top < (window.pageYOffset + window.innerHeight) &&
            left < (window.pageXOffset + window.innerWidth) &&
            (top + height) > window.pageYOffset &&
            (left + width) > window.pageXOffset
        );
    }

}

Configuration(function(ContextRegister) {
    ContextRegister.register('acompanhamento_aprovacao', acompanhamento_aprovacao);
});



// FILE: ./assets/js/controllers/aprovacao_alcadasController.js
function aprovacao_alcadasController(ScreenService, MessageService, templateManager, General, aprovacao_alcadasService) {

    var operacao = '';
    
    this.loadTemplate = function() {
        var templateUrl = {
            package: "chart-bar",
            baseUrl: "./assets/templates/html/chart-bar.html"
        };
        window.templateUrls.push(templateUrl);
    };

    this.filtraAprovador = function(owner) {
        var parametros = {
            "CDFILIAL": this.formataFiltro(templateManager.container.getWidget("FILTRO").currentRow.UNIDADE),
            "NRCONFIGALCADA": this.formataFiltro(templateManager.container.getWidget("FILTRO").currentRow.ALCADA),
            "CDOPERADOR": getLocalVar('CDOPERADOR')
        };

        General.onClickDependentField(owner, parametros);
    };

    this.abrirDetalhes = function(owner, e) {
        var tabbed = owner.container.getWidget("DETALHES_TABBED");
        var widgetDetalhes = owner.container.getWidget("DETALHES");
        var widgetItens = owner.container.getWidget('ITEM_INCONSISTENTE');
        var widgetComent = owner.container.getWidget('COMENTARIO');

        ScreenService.openSwipe(tabbed);
        
        widgetDetalhes.currentRow.NRAUTOFORNE = e.item.NRDEMANDA;
        widgetDetalhes.currentRow.CDNMFILDEST = e.item.CDNMFILDEST;
        widgetDetalhes.currentRow.CDFILIAL = e.item.CDFILDEST;
        widgetDetalhes.currentRow.CDFILLANC = e.item.CDFILLANC;
        widgetDetalhes.currentRow.IDCLASSIFIC = e.item.IDCLASSIFIC;
        widgetDetalhes.currentRow.VRTOTALAF = e.item.VRTOTALAF;
        widgetDetalhes.currentRow.ALCADA = e.item.NMALCADA;
        widgetDetalhes.currentRow.IDAPROVADO = e.item.IDAPROVADO;
        widgetDetalhes.currentRow.VRAPROVACAO = e.item.VRAPROVCMP;
        widgetDetalhes.currentRow.APROVADOR = e.item.APROVADOR;
        widgetDetalhes.currentRow.DSMOTVAFALCADA = e.item.DSMOTVAFALCADA;
        widgetDetalhes.currentRow.DTPROGENTAF = e.item.DTPROGENTAF.split(' ').shift();
        widgetDetalhes.currentRow.NRINSJURFORN = e.item.NRINSJURFORN;
        widgetDetalhes.currentRow.NMFANTFORN = e.item.NMFANTFORN;
        widgetDetalhes.currentRow.NMRAZSOCFORN = e.item.NMRAZSOCFORN;
        widgetDetalhes.currentRow.NRCONFIGALCADA = e.item.NRCONFIGALCADA;
        widgetDetalhes.currentRow.NRAPROVALCADA = e.item.NRAPROVALCADA;
        
        this.setActionsVisibility(e);
        
        var cdOperador = getLocalVar('CDOPERADOR');

        widgetItens.dataSourceFilter = [
            {
                "name": "CDFILDEST",
                "operator": "=",
                "value": e.item.CDFILDEST
            },
            {
                "name": "CDFILLANC",
                "operator": "=",
                "value": e.item.CDFILLANC
            },
            {
                "name": "NRAUTOFORN",
                "operator": "=",
                "value": e.item.NRDEMANDA
            }
        ];

        widgetComent.dataSourceFilter = [
            {
                "name": "NRCONFIGALCADA",
                "operator": "=",
                "value": e.item.NRCONFIGALCADA
            },
            {
                "name": "CDFILDEST",
                "operator": "=",
                "value": e.item.CDFILDEST
            },
            {
                "name": "CDFILLANC",
                "operator": "=",
                "value": e.item.CDFILLANC
            },
            {
                "name": "NRDEMANDA",
                "operator": "=",
                "value": e.item.NRDEMANDA
            }
        ];
        widgetItens.reload();
        widgetComent.reload();
    };
    
    this.setActionsVisibility = function(e) {
        var param = e.item.IDAPROVADO == 'Reprovado' || e.item.IDAPROVADO == 'Aprovado' ? false : true;
        templateManager.container.getWidget('DETALHES').actions[1].isVisible = param;
        templateManager.container.getWidget('DETALHES').actions[2].isVisible = param;
        templateManager.container.getWidget('ITEM_INCONSISTENTE').actions[1].isVisible = param;
        templateManager.container.getWidget('ITEM_INCONSISTENTE').actions[2].isVisible = param;
        templateManager.container.getWidget('COMENTARIO').actions[1].isVisible = param;
        templateManager.container.getWidget('COMENTARIO').actions[2].isVisible = param;
    };

    this.voltar = function() {
        ScreenService.closeSwipe();
        templateManager.container.getWidget('TASKS').activate();
    };

    this.openJustify = function(operacao) {
        this.operacao = operacao;
        var justificativa = templateManager.container.getWidget('JUSTIFICATIVA');

        if (operacao == 'APROVAR') {
            justificativa.getField('JUSTIFICATIVA').validations = {};
            justificativa.getAction('salvarJustificativa').label = 'Continuar';
        }
        else {
            justificativa.getField('JUSTIFICATIVA').validations = {
                'required': true,
                'minLength': {
                    'length': 10
                }
            };
            justificativa.getAction('salvarJustificativa').label = 'Salvar';
        }
        justificativa.currentRow.JUSTIFICATIVA = '';
        ScreenService.openPopup(justificativa);
    };

    this.aprovaAlcadas = function(widget) {
        if (widget.isValid()) {
            aprovacao_alcadasService.aprovaAlcada(widget.container.getWidget('DETALHES').currentRow, this.operacao, widget.currentRow.JUSTIFICATIVA).then(
                function(res) {
                    ScreenService.showMessage(res.messages[0].message);
                    ScreenService.closePopup();
                    this.voltar();
                    templateManager.container.getWidget('TASKS').reload();
                }.bind(this),
                function(error) {
                    let message = "wkf#ERRO_DURANTE_OPERACAO";
                    
                    if (error && error.error) {
                        message = 'wkf#ERR_GENERICO';
                    }
                    
                    MessageService.showMessageByCode(message, { 'PARAM': error.error });
                }
            );
        }
    };

    this.cancelar = function(widget) {
        ScreenService.closePopup();
        widget.activate();
    };

    this.filtrar = function() {
        setTimeout(function() {
            ScreenService.openPopup(templateManager.container.getWidget("FILTRO"));
        }, 350);
    };
    
    this.setDefaultValue = function() {
        const filterWidget = templateManager.container.getWidget("FILTRO");
        const date = new Date();
        const firstDay = new Date(date.getFullYear(), date.getMonth(), 1);
        const lastDay = new Date(date.getFullYear(), date.getMonth() + 1, 0);
        
        filterWidget.getField('DTINCLUSAO').defaultValue = {
			'formattedStart' : moment(firstDay).format('DD/MM/YYYY'),
			'start' : moment(firstDay).format('DD/MM/YYYY'),
			'formattedEnd' : moment(lastDay).format('DD/MM/YYYY'),
			'end' : moment(lastDay).format('DD/MM/YYYY')
		};
        filterWidget.applyDefaultValue();
    };

    this.aplicarFiltro = function(widget) {
        if (widget.container.getWidget("FILTRO").isValid()) {
            var widTask = widget.container.getWidget('TASKS');
            var widStripe = widget.container.getWidget('STRIPE');
            widTask.dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": this.formataFiltro(widget.currentRow.UNIDADE)
                },
                {
                    "name": "NRCONFIGALCADA",
                    "operator": "=",
                    "value": this.formataFiltro(widget.currentRow.ALCADA)
                },
                {
                    "name": "CDOPERADOR",
                    "operator": "=",
                    "value": widget.currentRow.CDOPERADOR
                },
                {
                    "name": "IDCLASSIFIC",
                    "operator": "=",
                    "value": this.formataFiltro(widget.currentRow.IDCLASSIFIC)
                },
                {
                    "name": "IDAPROVADO",
                    "operator": "=",
                    "value": widget.currentRow.IDAPROVADO
                },
                {
                    "name": "DTINCLUSAO",
                    "operator": "=",
                    "value": widget.currentRow.DTINCLUSAO
                },
                {
                    "name": "DTENTREGAAF",
                    "operator": "=",
                    "value": widget.currentRow.DTENTREGAAF
                }
            ];
            ScreenService.closePopup();
            widStripe.currentRow.APROVADOR_STRIPE = 'Aprovador Principal: ' + widget.currentRow.APROVADOR;
            widTask.activate();
            widTask.reload();
        }
    };

    this.formataFiltro = function(celula) {
        var v = ((Array.isArray(celula)) && (celula.length > 0)) ? celula.join('_') : 'T';
        return v;
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('aprovacao_alcadasController', aprovacao_alcadasController);
});

// FILE: ./assets/js/controllers/configuracao_AlcadasController.js
function configuracao_AlcadasController(ScreenService, templateManager, configuracao_AlcadasService, General, MessageService, ZHPromise) {

    const DEFAULT_FIELDS_CONFIG_ALCADA = [ 'NMALCADA', 'IDTPALCADA', 'IDLANCAFEXTRA', 'IDLANCSCEXTRA', 'IDTIPOAF', 'IDTIPOSC', 'IDTPLANC','IDTIPOVRAPROV' ];
    const VALOR_APROVACAO_DEFAULT = 0;
    const nameOfValueFields = [ 'VRDIRETORIA', 'VRGERREGIONAL', 'VRPLANEJAMENTO', 'VRSUPERVISOR' ];
    const dependentFieldsAndTheisParents = { 'IDVALIDAALTER': 'IDALTERACAO' };
    const fieldsPorTipoAlcada = {
        'ORCAMENTO': [ 'IDTIPORCAOP', 'NMDESPPREV', 'IDFORAORC', 'IDALTERACAO', 'VALIDACAO', 'IDEXIGETAPROVE', 'IDIMPGESTFIL' ],
        'COMPRAS': [ 'IDCANCEL', 'IDBAIXMANUAL', 'IDEXCLUSAO', 'IDALTERACAO', 'VALIDACAO', 'IDEXIGETAPROVE', 'IDIMPGESTFIL', 'NMOPERADOR' ],
        'FINANCEIRO': [ 'DSTPCTPAGAR', 'IDEXIGETAPROVE', 'NMSETOR' ],
        'CATALOGO': [ 'IDTIPORCAOP', 'NMDESPPREV', 'IDFORAORC', 'VALIDACAO', 'IDEXIGETAPROVE', 'IDIMPGESTFIL' ]
    };
    const codigosPorTipoAlcada = {
        'ORCAMENTO': [ 'CDDESPPREV' ],
        'COMPRAS': [ 'CDOPERADOR' ],
        'FINANCEIRO': [ 'CDTPCTPAGAR', 'CDSETOR' ],
        'CATALOGO': [ 'CDDESPPREV' ],
    };
    let mailMessage;
    const self = this;
    
    /******************
     * FUNÇÕES GERAIS *
     * ****************/

    this.widgetNewRow = function(widget) {
        const enableWidget = false;
        
        enableParentWidgets(widget, enableWidget);
        widget.newRow();
        this.setForeignKeyOnChildWidget(widget);
    };

    this.widgetEditOnMasterDetail = function(widget) {
        const enableWidget = false;
        
        this.createCurrentRowBackupOnWidget(widget);
        this.createUpdateParameterOnWidgetCurrentRow(widget);
        enableParentWidgets(widget, enableWidget);
        widget.edit();
    };

    this.widgetCancelFormOnMasterDetail = function(widget) {
        const enableWidget = true;
        
        if (widget.currentRow.__is_new) {
            ScreenService.closeSwipe();
            widget.currentRow = {};
            widget.list();
            widget.dataSource.data.pop();
        } else {
            this.returnBackupRowOnCurrentWidget(widget);
            widget.view();
        }
        
        enableParentWidgets(widget, enableWidget);
    };
    
    function getMessageFromResponse(response) {
        return response.length !== 0 ? response.messages[0].message : undefined;
    }
    
    function getCleanErrorMessage(errorResponse) {
        return MessageService.showMessageByCode("wkf#".concat(errorResponse.data.error.replace('Exception:', '')), {}, {
            type: 'ERROR'
        });
    }
    
    function saveRowOnForm(widget, route) {
        removeActionsFromRow(widget);
        
        configuracao_AlcadasService.requestRow(widget.currentRow, route).then(
            function(response) {
                afterSaveSwipe(widget, getMessageFromResponse(response));
            }
        ).catch(
            function(error) {
                getCleanErrorMessage(error);
            }
        );
    }

    function deleteRowOnMasterDetail(widget, route) {
        MessageService.confirmMessageByCode("wkf#CONFIRMA_EXCLUSAO", 'question').then(function(success) {
            configuracao_AlcadasService.requestRow(widget.currentRow, route).then(
                function(response) {
                    const isExclusao = true;
                    
                    afterSaveSwipe(widget, getMessageFromResponse(response), isExclusao);
                }
            ).catch(
                function(error) {
                    getCleanErrorMessage(error);
                }
            );
        }, function(error) {
            return false;
        });
    }

    this.returnBackupRowOnCurrentWidget = function(widget) {
        Object.keys(widget.backupRow).forEach(function(key) {
            widget.currentRow[key] = widget.backupRow[key];
        });
    };

    this.createCurrentRowBackupOnWidget = function(widget) {
        widget.backupRow = angular.copy(widget.currentRow);
    };

    this.createUpdateParameterOnWidgetCurrentRow = function(widget) {
        widget.currentRow.__is_new = false;
    };

    function enableParentWidgets(widget, enableWidget) {
        widget.widgets.forEach(function(wid) {
            wid.enabled = enableWidget;
        });
    }
    
    function isContainer(widget) {
        return widget.getParent() == null;
    }

    function removeActionsFromRow(widget) {
        delete widget.currentRow.__leftActions;
        delete widget.currentRow.__rightActions;
        
        if (widget.getParent() && !isContainer(widget.getParent())) {
            delete widget.getParent().currentRow.__leftActions;
            delete widget.getParent().currentRow.__rightActions;
        }
    }

    function afterSavePoppup(widget, message) {
        const isExclusao = false;
        
        ScreenService.closePopup();
        
        if (isContainer(widget)) {
            widget.reload();
        } else {
            widget.getParent().activate();
        }
        
        widget.currentRow = {};
        exibeMensagemOperacaoConcluida(message, isExclusao);
    }

    function afterSaveSwipe(widget, message, isExclusao = false) {
        exibeMensagemOperacaoConcluida(message, isExclusao);
        closeSwipeAndEnableParentWidgets(widget);
    }
    
    function closeSwipeAndEnableParentWidgets(widget) {
        const enableWidget = true;
        
        ScreenService.closeSwipe().then(function() {
            widget.activate();
            widget.reload();
        });
        
        enableParentWidgets(widget, enableWidget);
    }
    
    function exibeMensagemOperacaoConcluida(responseMessage, isExclusao = false) {
        const mensagemSucessoOperacao = isExclusao ? 'EXCLUIDO_SUCESSO' : 'OPERACAO_SUCESSO';
        
        if (!Util.isEmptyOrBlank(responseMessage)) {
            if (Array.isArray(responseMessage)) {
                exibeLog(responseMessage);
            } else {
                MessageService.showMessageByCode('wkf#'.concat(responseMessage), {}, {
                    type: 'INFORMATION'
                });
            }
        }
        
        MessageService.successNotificationByCode("wkf#".concat(mensagemSucessoOperacao));
    }

    this.setForeignKeyOnChildWidget = function(widget) {
        const childRow = widget.currentRow;
        const parentRow = widget.getParent().currentRow;
        
        switch (widget.name) {
            case 'unidades':
                childRow.NRCONFIGALCADA = parentRow.NRCONFIGALCADA;
                break;

            case 'aprovadores':
                childRow.NRCONFIGALCADA = parentRow.NRCONFIGALCADA;
                childRow.CDFILIAL = parentRow.CDFILIAL;
                break;

            case 'aprovadores_relacionados':
                childRow.NRCONFIGALCADA = parentRow.NRCONFIGALCADA;
                childRow.CDFILIAL = parentRow.CDFILIAL;
                // childRow.CDOPERADOR = parentRow.CDOPERADOR;
                break;
        }
    };
    
    function retornaParametroEnvioQrcode(widget, name, montaLista) {
        let checkedRows = widget.getCheckedRows();
        let valor = '';
        
        if(checkedRows.length) {
            
            if(montaLista) {
                valor = checkedRows.map(function(row) {
                    return row[name];
                }).filter(function(row) {
                    return row;
                }).join('_');
            } else {
                valor = checkedRows[0][name];
            }
            
        } else {
            valor = widget.currentRow[name];
        }
        
        return valor ? valor : 'T';
        
    }

    function getValueFromString(value) {
        if (Util.isEmptyOrBlank(value)) {
            return VALOR_APROVACAO_DEFAULT;
        }
        
        if (typeof value === 'string') {
            value = value.replace('R$', '');
            value = value.replace('.', ''); 
            value = value.replace(',', '.');
        }
        
        return parseFloat(value);
    }
    
    function getEvent(item, name) {
        return item.events.filter( event => event.name === name ).shift();
    }
    
    /****************************************
     * CADASTRAMENTO AUTOMÁTICO DE UNIDADES *
     ****************************************/
     
    this.cadastraUnidadeAuto = function() {
        const widgetCadAutomaticoUnidade = templateManager.container.getWidget('cadastro_automatico_unidade');
        
        if (widgetCadAutomaticoUnidade.isValid()) {
            removeActionsFromRow(widgetCadAutomaticoUnidade);
            
            configuracao_AlcadasService.saveFilialAuto(widgetCadAutomaticoUnidade.currentRow).then(
                function(response) {
                    afterSavePoppup(widgetCadAutomaticoUnidade);
                    
                    return response;
                }
            )
            .catch(response => response)
            .then(function(response) {
                const messages = response.data.messages;
                
                if (messages) {
                    let actionsModified = [
                        {
                            label: ScreenService.i18n("Ok"),
                            "default": true,
                            code: function(e, deferred) {
                                deferred.resolve(e);
                            }
                        }
                    ];
                    
                    MessageService.getMessageByCode('wkf#DETAILS').then(function(callback) {
                        if (Array.isArray(response.data.messages) && !Util.isEmptyOrBlank(response.data.messages)) {
                            actionsModified.push({
                                label: callback,
                                code: function(e, deferred) {
                                    deferred.reject(exibeLog(messages[0].message));
                                }
                            });
                        }
                        MessageService.showCustomDialogByCode('wkf#'.concat(response.data.error), null, {
                            type: 'INFORMATION',
                            actions: actionsModified
                        });
                    });
                }
            });
        }
    };


    /******************************************************
     * CONFIGURAÇÃO DAS ALÇADAS                           *
     ******************************************************/
    
    this.newRowConfigAlcada = function() {
        const widgetConfigAlcada = templateManager.container.getWidget('configuracao_das_alcadas');
        
        redefinirRegrasAlcada();
        
        this.widgetNewRow(widgetConfigAlcada);
    };
    
    function filtrarRowPorTipoAlcada(currentRow) {
        const tipoAlcada = currentRow.IDTPALCADA;
        const isPropriedadeDaAlcada = function(nomePropriedade) {
            return DEFAULT_FIELDS_CONFIG_ALCADA.includes(nomePropriedade) ||
                    fieldsPorTipoAlcada[tipoAlcada].includes(nomePropriedade) ||
                    nameOfValueFields.includes(nomePropriedade) ||
                    codigosPorTipoAlcada[tipoAlcada].includes(nomePropriedade) ||
                    !!dependentFieldsAndTheisParents[nomePropriedade];
        };
        let newCurrentRow = Object.keys(currentRow)
            .filter(isPropriedadeDaAlcada)
            .reduce( (res, key) => Object.assign(res, { [key]: currentRow[key] }), {} );
        
        newCurrentRow.NRCONFIGALCADA = currentRow.NRCONFIGALCADA;
        
        return newCurrentRow;
    }
    
    this.saveConfigAlcada = function() {
        const widgetConfigAlcada = templateManager.container.getWidget('configuracao_das_alcadas');
        const executaSalvarConfiguracaoAlcada = function() {
            const tipoAlcadaSelecionada = getTipoAlcadaSelecionada();
            const fieldsNaoAfetados = getFieldsNaoAfetados(tipoAlcadaSelecionada);
            let rowForRequest;
            let rota;
            
            fieldsNaoAfetados.filter(field => !Util.isEmptyOrBlank(field.value())).forEach(field => field.clearValue());
            
            if (widgetConfigAlcada.currentRow.__is_new) {
                rowForRequest = widgetConfigAlcada.currentRow;
                rota = 'configuracao_das_alcadas/save_new';
            } else {
                const currentRow = filtrarRowPorTipoAlcada(widgetConfigAlcada.currentRow);
                const backupRow = filtrarRowPorTipoAlcada(widgetConfigAlcada.backupRow);
                
                rowForRequest = {
                    NEW: currentRow,
                    OLD: backupRow
                };
                rota = 'configuracao_das_alcadas/save_edit';
            }
            
            removeActionsFromRow(widgetConfigAlcada);
            configuracao_AlcadasService.requestRow(rowForRequest, rota).then(
                function(response) {
                    afterSaveSwipe(widgetConfigAlcada, getMessageFromResponse(response));
                }
            ).catch(
                function(error) {
                    getCleanErrorMessage(error);
                }
            );
        };
        const verificaAlcadaRepetida = function(){
            let alcadaUnica = 0;
            widgetConfigAlcada.dataSource.data.forEach(function(data) {
                if (data.NMALCADA == widgetConfigAlcada.currentRow.NMALCADA && data.IDTPALCADA == widgetConfigAlcada.currentRow.IDTPALCADA) {
                    alcadaUnica++;
                }
            });
            return alcadaUnica <= 1 ? true:false;
        };
        
        if (widgetConfigAlcada.isValid()) {
            if(verificaAlcadaRepetida()){
                executaSalvarConfiguracaoAlcada();
            } else {
                MessageService.showMessageByCode("wkf#ALCADA_CADASTRADA", {}, {
                    type: 'INFORMATION'
                });
            }
        } 
    };

    this.editConfigAlcada = function() {
        const widgetConfigAlcada = templateManager.container.getWidget('configuracao_das_alcadas');
        let isReadOnlyOnEdit = false;
        
        if (possuiAprovacoesPendentes(widgetConfigAlcada)) {
            isReadOnlyOnEdit = true;
            MessageService.showMessageByCode("wkf#ONLY_NAME_CAN_BE_CHANGED", {}, {
                type: 'INFORMATION'
            });
        }
        
        widgetConfigAlcada.fields.forEach(function(field) {
            if (field.name !== 'NMALCADA') {
                field.readOnlyOnEdit = isReadOnlyOnEdit;
            }
        });
        
        this.setControlOfFieldDespesasDiversas();
        this.widgetEditOnMasterDetail(widgetConfigAlcada);
    };
    
    this.deleteConfigAlcada = function() {
        const widgetConfigAlcada = templateManager.container.getWidget('configuracao_das_alcadas');
        
        if (possuiAprovacoesPendentes(widgetConfigAlcada)) {
            MessageService.showMessageByCode("wkf#OPERATION_BLOCKED_DEL_COMPETENCE_PENDING_APPROVALS", {}, {
                type: 'INFORMATION'
            });
        } else {
            deleteRowOnMasterDetail(widgetConfigAlcada, 'configuracao_das_alcadas/remove');
        }
    };
    
    this.cancelarConfigAlcada = function() {
        const widgetConfigAlcada = templateManager.container.getWidget('configuracao_das_alcadas');
        
        this.widgetCancelFormOnMasterDetail(widgetConfigAlcada);
        
        if (!widgetConfigAlcada.currentRow.__is_new) {
            aplicarRegrasAlcadaModoEdicao(widgetConfigAlcada);
        }
    };
    
    this.voltarConfigAlcada = function() {
        const widgetConfigAlcada = templateManager.container.getWidget('configuracao_das_alcadas');
        
        widgetConfigAlcada.currentRow = {};
        ScreenService.closeSwipe();
    };
    
    this.onEnterOnceConfiguracaoAlcada = function() {
        initialValidation();
        checkIdSisDos();
    };
    
    function checkIdSisDos() {
        configuracao_AlcadasService.requestRow({}, 'getIdSisDos').then(
            function(response) {
                const idSisDos = response.dataset.callback.IDSISDOS;
                const widget = templateManager.container.getWidget("configuracao_das_alcadas");
                
                widget.getAction("UPLOAD_SPREADSHEET").isVisible = idSisDos == 3;
            }
        );
    }
    
    function initialValidation() {
        const filter = [
            {
                name: 'FILIALLOGADA',
                operator: '=',
                value: getLocalVar("CDFILIAL")
            }
        ];

        configuracao_AlcadasService.getInitializingParameter(filter).then(function(response) {
                const parametrizacao = response.dataset.parametrizacao;
                
                if (!Util.isEmptyOrBlank(parametrizacao)) {
                    const verificaOrcamentoUnidade = parametrizacao[0].IDVERIORCFILI;
                    
                    lockFieldsWithParameter(verificaOrcamentoUnidade);
                }
            }
        );
    }
    
    function getFieldsNameToLock(verificaOrcamentoUnidade) {
        let fieldsToLock = false;
        
        switch (verificaOrcamentoUnidade) {
            case 'VALIDA_SOL':
                fieldsToLock = [ 'IDLANCAFEXTRA', 'IDTIPOAF' ];
                break;

            case 'VALIDA_AF':
                fieldsToLock = [ 'IDLANCSCEXTRA', 'IDTIPOSC' ];
                break;
                
            case 'VALIDA_PED':
                // widgetConfigAlcada.getField('IDLANCSCEXTRA').readOnly = true;
                // widgetConfigAlcada.getField('IDTIPOSC').readOnly = true;
                // widgetConfigAlcada.getField('IDTIPOAF').readOnly = true;
                break;

            case 'VALIDA_SOL_AF':
                fieldsToLock = [ 'IDLANCSCEXTRA', 'IDTIPOSC', 'IDTIPOAF' ];
                break;
                
            case 'VALIDA_SOL_PED':
                // widgetConfigAlcada.getField('IDLANCSCEXTRA').readOnly = true;
                // widgetConfigAlcada.getField('IDTIPOSC').readOnly = true;
                // widgetConfigAlcada.getField('IDTIPOAF').readOnly = true;
                break;
                
            case 'VALIDA_SOL_AF_PED':
                // widgetConfigAlcada.getField('IDLANCSCEXTRA').readOnly = true;
                // widgetConfigAlcada.getField('IDTIPOSC').readOnly = true;
                // widgetConfigAlcada.getField('IDTIPOAF').readOnly = true;
                break;
            case 'VALIDA_AF_PED':
                // widgetConfigAlcada.getField('IDLANCSCEXTRA').readOnly = true;
                // widgetConfigAlcada.getField('IDTIPOSC').readOnly = true;
                // widgetConfigAlcada.getField('IDTIPOAF').readOnly = true;
                break;
        }
        
        return fieldsToLock;
    }
    
    function lockFieldsWithParameter(verificaOrcamentoUnidade) {
        if (verificaOrcamentoUnidade == 'NAO_VALIDA') {
            return travaFieldsSeNaoValidaWorkflow();
        }
        
        const fieldsConfigAlcada = templateManager.container.getWidget('configuracao_das_alcadas').fields;
        const fieldsName = getFieldsNameToLock(verificaOrcamentoUnidade);
        
        if (fieldsName) {
            const isFieldToLock = field => fieldsName.includes(field.name);
            const travarField = field => field.readOnly = true;
            
            fieldsConfigAlcada.filter(isFieldToLock).forEach(travarField);
        }
    }

    function travaFieldsSeNaoValidaWorkflow() {
        MessageService.getMessageByCode('wkf#OPERATION_BLOCKED_WF_N_PARAMETRIZED').then(function(message) {
            const msg = "ScreenService.showMessage('".concat(message, "');");
            const widgetsToUpdateAction = [ 'configuracao_das_alcadas', 'unidades', 'aprovadores', 'aprovadores_relacionados' ];
            const actionsToFind = [ 'add', 'edit', 'delete', 'cadastramento_automatico', 'enviarQrcode' ];
            const isActionToLock = function(action) {
                return actionsToFind.includes(action.name);
            };
            const updateActionEventCode = function(action) {
                getEvent(action, 'ActionEvent').code = msg;
            };
            const getWidgetAndTriggerUpdateActionEventCode = function(widgetName) {
                templateManager.container.getWidget(widgetName).actions.filter(isActionToLock).forEach(updateActionEventCode);
            };
            
            widgetsToUpdateAction.forEach(getWidgetAndTriggerUpdateActionEventCode);
        });
    }
    
    this.setControlOfFieldDespesasDiversas = function() {
        const widgetConfigAlcada = templateManager.container.getWidget('configuracao_das_alcadas');
        const fieldDespesasDiversas = widgetConfigAlcada.getField('NMDESPPREV');
        const itemOrcamento = widgetConfigAlcada.getField('IDTIPORCAOP').value();
        
        if (itemOrcamento === 'CONSUMO') {
            fieldDespesasDiversas.value(null);
            fieldDespesasDiversas.validations = {};
            fieldDespesasDiversas.readOnly = true;
        } else {
            fieldDespesasDiversas.readOnly = false;
            const tipoAlcada = getTipoAlcadaSelecionada();
            
            if (tipoAlcada !== "CATALOGO") {
                fieldDespesasDiversas.validations = {
                    required: true
                };
            }
        }
    };
    
    function possuiAprovacoesPendentes(widget) {
        return widget.currentRow.APROVACAO > 0;
    }
    
    function importarGestores() {
        const deveImportarGestores = templateManager.container.getWidget('configuracao_das_alcadas').getField('IDIMPGESTFIL').value();
        
        return deveImportarGestores === 'S';
    }
    
    function setFieldVisible(field) {
        field.isVisible = true;
    }
    
    function setFieldNotVisible(field) {
        field.isVisible = false;
    }
    
    function setFieldNotVisibleAndClearValue(field) {
        field.isVisible = false;
        field.clearValue();
    }
    
    function getFunctionToApplyOnFieldsValor(widgetConfigAlcada, applyDefaultValue = false) {
        if (importarGestores()) {
            const exigeAprovacao = exigeAprovacaoTodosAprovadores(widgetConfigAlcada);
            
            return function(field) {
                setFieldVisible(field);
                toggleFieldsOfApprovalValueAndOrder(field, exigeAprovacao, applyDefaultValue);
            };
        }
        
        return setFieldNotVisibleAndClearValue;
    }
    
    function aplicarRegrasAlcadaModoEdicao(widgetConfigAlcada) {
        const functionToApplyOnFieldValor = getFunctionToApplyOnFieldsValor(widgetConfigAlcada);
        const redefinirRegras = false;
        const forEachFieldsFunctions = {
            afetados: setFieldVisible,
            naoAfetados: setFieldNotVisible,
            valor: functionToApplyOnFieldValor,
        };
        
        aplicarRegrasAlcada(forEachFieldsFunctions, redefinirRegras);
    }
    
    this.widgetConfigAlcadaAfterMoveRow = function() {
        const widgetConfigAlcada = templateManager.container.getWidget('configuracao_das_alcadas');
        const widgetAprovadores = templateManager.container.getWidget('aprovadores');
        
        aplicarRegrasAlcadaModoEdicao(widgetConfigAlcada);
        ScreenService.closeSwipe(1);
    };
    
    this.onChangeImportarGestoresUnidade = function() {
        const widgetConfigAlcada = templateManager.container.getWidget('configuracao_das_alcadas');
        const applyDefaultValue = true;
        const functionToApplyOnFieldValor = getFunctionToApplyOnFieldsValor(widgetConfigAlcada, applyDefaultValue);
        
        getValueFields(widgetConfigAlcada).forEach(functionToApplyOnFieldValor);
    };
    
    function isFieldValor(field) {
        return nameOfValueFields.includes(field.name);
    }
    
    function getFieldNameOfTipoAlcadasDiferenteDaSelecionada(tipoAlcadaSelecionada) {
        let fieldName = [];
        const tipoAlcadaNaoAfetada = Object.keys(fieldsPorTipoAlcada).filter(tipoAlcada => tipoAlcada !== tipoAlcadaSelecionada);
        const isFieldNaoAfetado = function(name) {
             return !fieldsPorTipoAlcada[tipoAlcadaSelecionada].includes(name);
        };
        const getNomeFieldsNaoAfetados = function(tipoAlcadaNaoAfetada) {
            return fieldsPorTipoAlcada[tipoAlcadaNaoAfetada].filter(isFieldNaoAfetado);
        };
        
        tipoAlcadaNaoAfetada.forEach(tipoAlcadaNaoAfetada => {
            fieldName = [ ...fieldName, ...getNomeFieldsNaoAfetados(tipoAlcadaNaoAfetada) ];
        });
        
        return fieldName;
    }
    
    function getFieldsNaoAfetados(tipoAlcadaSelecionada) {
        const nameOfFieldsNaoAfetados = getFieldNameOfTipoAlcadasDiferenteDaSelecionada(tipoAlcadaSelecionada);
        
        return templateManager.container.getWidget('configuracao_das_alcadas').fields.filter( field => nameOfFieldsNaoAfetados.includes(field.name) );
    }
    
    function getValueFields(widgetConfigAlcada) {
        return widgetConfigAlcada.fields.filter(isFieldValor);
    }
    
    function getFieldsAlcada(widgetConfigAlcada, tipoAlcadaSelecionada) {
        const nameOfFieldsNaoAfetados = getFieldNameOfTipoAlcadasDiferenteDaSelecionada(tipoAlcadaSelecionada);
        const isFieldAfetado = function(field) { return fieldsPorTipoAlcada[tipoAlcadaSelecionada].includes(field.name); };
        const isFieldNaoAfetado = function(field) { return nameOfFieldsNaoAfetados.includes(field.name); };
        const isFieldDependent = function(field) { return !!dependentFieldsAndTheisParents[field.name]; };
        let fields = { afetados: [], naoAfetados: [], valor: [], dependentes: [] };
        
        widgetConfigAlcada.fields.forEach(function(field) {
            if (isFieldAfetado(field)) {
                fields.afetados.push(field);
            } else if (isFieldNaoAfetado(field)) {
                fields.naoAfetados.push(field);
            } else if (isFieldValor(field)) {
                fields.valor.push(field);
            } else if (isFieldDependent(field)) {
                fields.dependentes.push(field);
            }
        });
        
        return fields;
    }
    
    function getTipoAlcadaSelecionada() {
        return templateManager.container.getWidget('configuracao_das_alcadas').currentRow.IDTPALCADA;
    }
    
    function redefinirRegrasAlcada() {
        const widgetConfigAlcada = templateManager.container.getWidget('configuracao_das_alcadas');
        
        widgetConfigAlcada.fields.forEach(field => {
            if (DEFAULT_FIELDS_CONFIG_ALCADA.includes(field.name)) {
                field.isVisible = true;
                field.applyDefaultValue();
            } else {
                field.isVisible = false;
            }
        });
    }
    
    function getForEachFieldsFunctions(forEachFieldsFunctions) {
        const widgetConfigAlcada = templateManager.container.getWidget('configuracao_das_alcadas');
        const exigeAprovacao = exigeAprovacaoTodosAprovadores(widgetConfigAlcada);
        const dependentesFunction = function(childField) {
            const isCheckboxMarcado = function(checkboxField) { return checkboxField.value() == 'S'; };
            let parentField = widgetConfigAlcada.getField(dependentFieldsAndTheisParents[childField.name]);
            
            if (parentField.isVisible) {
                childField.isVisible = isCheckboxMarcado(parentField);
            } else {
                childField.isVisible = false;
            }
        };
        
        if (!forEachFieldsFunctions) {
            const newForEachFieldsFunctions = {
                afetados: function(field) {
                    field.isVisible = true; field.applyDefaultValue();
                },
                naoAfetados: function(field) {
                    field.isVisible = false;
                },
                valor: function(field) {
                    resetFieldsOfApprovalValueAndOrder(field); toggleFieldsOfApprovalValueAndOrder(field, exigeAprovacao);
                },
                dependentes: dependentesFunction,
            };
            
            return newForEachFieldsFunctions;
        }
        
        if (!forEachFieldsFunctions.dependentes) {
            forEachFieldsFunctions.dependentes = dependentesFunction;
        }
        
        return forEachFieldsFunctions;
    }
    
    function setLabel(object, codeMsg) {
        MessageService.getMessageByCode('wkf#'.concat(codeMsg)).then(function(message) {
            object.label = message;
        });
    }
    
    function aplicarRegrasAlcada(forEachFieldsFunctions, redefinirRegras = true) {
        const tipoAlcadaSelecionada = getTipoAlcadaSelecionada();
        
        if (Util.isEmptyOrBlank(tipoAlcadaSelecionada)) {
            if (redefinirRegras) {
                redefinirRegrasAlcada();
            }
        } else {
            forEachFieldsFunctions = getForEachFieldsFunctions(forEachFieldsFunctions);
            
            const widgetConfigAlcada = templateManager.container.getWidget('configuracao_das_alcadas');
            const { afetados: fieldsAfetados, naoAfetados: fieldsNaoAfetados, valor: fieldsValor, dependentes: fieldsDependentes } = getFieldsAlcada(widgetConfigAlcada, tipoAlcadaSelecionada);
            const fieldDespesasDiversas = widgetConfigAlcada.getField('NMDESPPREV');
            
            fieldsAfetados.forEach(forEachFieldsFunctions.afetados);
            fieldsNaoAfetados.forEach(forEachFieldsFunctions.naoAfetados);
            fieldsValor.forEach(forEachFieldsFunctions.valor);
            fieldsDependentes.forEach(forEachFieldsFunctions.dependentes);
            
            switch (tipoAlcadaSelecionada) {
                case 'ORCAMENTO':
                    if (widgetConfigAlcada.getMode() === "edit") {
                        fieldDespesasDiversas.clearValue();
                    }
                    
                    fieldDespesasDiversas.readOnly = true;
                    fieldDespesasDiversas.validations = {};
                    
                    self.setVisibilityDefaultFieds(widgetConfigAlcada, true);
                    break;
                case 'COMPRAS':
                    setLabel(widgetConfigAlcada.getField('IDIMPGESTFIL'), "IMPORT_BUYER_MANAGER");
                    self.setVisibilityDefaultFieds(widgetConfigAlcada, true);
                    break;
                case 'CATALOGO':
                    if (widgetConfigAlcada.getMode() === "edit") {
                        fieldDespesasDiversas.clearValue();
                    }
                    
                    fieldDespesasDiversas.readOnly = true;
                    fieldDespesasDiversas.validations = {};
                    
                    self.setVisibilityDefaultFieds(widgetConfigAlcada, false);
                    break;
                default:
                    self.setVisibilityDefaultFieds(widgetConfigAlcada, true);
                    break;
            }
            
            if (tipoAlcadaSelecionada !== 'COMPRAS') {
                setLabel(widgetConfigAlcada.getField('IDIMPGESTFIL'), "IMPORT_UNITS_MANAGER");
            }
        }
    }
    
    this.setVisibilityDefaultFieds = function(widget, isVisible) {
        widget.getField('IDLANCAFEXTRA').isVisible = isVisible;
        widget.getField('IDTIPOVRAPROV').isVisible = isVisible;
        widget.getField('IDLANCSCEXTRA').isVisible = isVisible;
        widget.getField('IDTIPOAF').isVisible = isVisible;
        widget.getField('IDTIPOSC').isVisible = isVisible;
        widget.getField('IDTPLANC').isVisible = isVisible;
    };
    
    this.onChangeTipoAlcada = function() {
        aplicarRegrasAlcada();
    };
    
    this.aplicarRegrasExigeAprovacaoTodosAprovadores = function() {
        const widgetConfigAlcada = templateManager.container.getWidget('configuracao_das_alcadas');
        const exigeAprovacao = exigeAprovacaoTodosAprovadores(widgetConfigAlcada);
        const valueFields = getValueFields(widgetConfigAlcada);
        
        valueFields.forEach(field => 
                toggleFieldsOfApprovalValueAndOrder(field, exigeAprovacao)
            );
        
        const applyDefaultValue = exigeAprovacao ? false : true;
        
        if (applyDefaultValue) {
            /**
             * Força a página a ser redesenhada par atualizar o template, 
             * evitando assim o erro "Expected `R$0,00` to be a number"
             * ao aplicar o valor default.
             * 
             * Necessário por conta da transição entre templates realizada
             * pela 'toggleFieldsOfApprovalValueAndOrder'.
             * 
             * Fonte: https://code.angularjs.org/1.3.20/docs/error/ngModel/numfmt?p0=R$0,00
             */
            widgetConfigAlcada.redraw();
        
            valueFields.forEach(field => field.applyDefaultValue());
        }
    };
    
    function resetFieldsOfApprovalValueAndOrder(field) {
        field.isVisible = false;
        field.validations = {};
        field.clearValue();
    }
    
    function exigeAprovacaoTodosAprovadores(widgetConfigAlcada) {
        const exigeAprovacaoAprovadores = widgetConfigAlcada.getField('IDEXIGETAPROVE').value();
        
        return exigeAprovacaoAprovadores === 'S';
    }
    
    function toggleFieldsOfApprovalValueAndOrder(field, exigeAprovacao, applyDefaultValue = false, tipoAprovacao = 'VALOR') {
        normalizeValue(field);
        
        if (exigeAprovacao) {
            field.template = 'field/number.html';
            field.minValue = 1;
            field.spin = true;
            
            delete field.maxlength;
            delete field.mask;
        } else {
            if (tipoAprovacao == 'VALOR') {
                    field.template = 'field/text-edit.html';
                    field.maxlength = 18;
                    field.mask = {
                        type: 'currency',
                        params: {
                            allowNegative: false,
                            symbol: 'R$',
                            thousands: '.',
                            decimal: ',',
                            precision: '2'
                        }
                    };
            }else{
                    field.template = 'field/text-edit.html';
                    field.maxlength = 3;
                    field.mask = {
                        type: 'percent',
                        params: {
                            allowNegative: false,
                            symbol: '%'
                        }
                    };
                }
        }
        
        field.validations = {
            required: true
        };
        
        if (applyDefaultValue) {
            field.applyDefaultValue();
        }
    }
    
    function normalizeValue(field) {
        field.setValue(field.value() || 0);
    }
    
    this.onChangeCheckboxAlteracao = function() {
        const fieldValidaAlteracao = templateManager.container.getWidget('configuracao_das_alcadas').getField('IDVALIDAALTER');
        
        fieldValidaAlteracao.isVisible = !fieldValidaAlteracao.isVisible;
        
        if (fieldValidaAlteracao.isVisible) {
            fieldValidaAlteracao.applyDefaultValue();
        } else {
            fieldValidaAlteracao.clearValue();
        }
    };
    
    /******************************************************
     * FUNÇÔES TELA UNIDADE: MÚLTIPLOS APROVADORES (GR)   *
     ******************************************************/
    
    this.bindEventsMultiplosAprovadores = function(widget) {
        /**
         * Interval que irá iniciar ao abrir a tela de cadastrar Unidades com Múltilplos Aprovadores (Catálogo/GR).
         * 
         * Ele irá executar enquanto o Swipe estiver aberto, forçando o click nos campos select-autocomplete abrir
         * o PopUp, para evitar problemas que o template está apresentando.
         * 
         * Condição para remover o interval: Fechar o swipe.
         */
        let interval = setInterval(function() {
            for (let i = 1; i <= 4; i++) {
                if ($('div.zh-field-NMOPERADOR' + i).siblings('div.list-items.show-select').css('display') != "none") {
                    $('div.zh-field-NMOPERADOR' + i).siblings('div.list-items.show-select').css('display', 'none');
                    $('div#NMOPERADOR' + i + ' span svg').mousedown();
                }
                
                if (widget.shouldClearInterval) {
                    self.shouldClearInterval(widget, false);
                    clearInterval(interval);
                }
            }
        }, 200);
    };
    
    this.shouldClearInterval = function(widget, value) {
        widget.shouldClearInterval = value;
    };
     
    this.toggleVisibilityMultiplosAprovadores = function(widgetUnidades) {
        const widgetAlcada = widgetUnidades.container.getWidget('configuracao_das_alcadas');
        const tipoAlcada = widgetAlcada.currentRow.IDTPALCADA;
        const isCatalogo = (tipoAlcada == 'CATALOGO');
        widgetUnidades.isTypeCatalogo = isCatalogo;

        const isImpGest = (widgetAlcada.currentRow.IDIMPGESTFIL == 'N');
        widgetUnidades.isImpGestUnidOff = isImpGest;
        
        if (isCatalogo && isImpGest) {
            configuracao_AlcadasService.requestRow({}, 'getIdSisDos').then(
                function(response) {
                    const idSisDos = response.dataset.callback.IDSISDOS;
                    
                    const isGr = idSisDos == "3" ? true : false;
                    widgetUnidades.isGR = isGr;
                    
                    self.setMultiplosAprovadoresVisibility(widgetUnidades, isGr);
                }
            );
        } else {
            self.setMultiplosAprovadoresVisibility(widgetUnidades, false);
        }
    };
    
    this.changeValueFieldTypeMultiplosAprovadores = function(widget) {
        const widgetConfigAlcada = templateManager.container.getWidget('configuracao_das_alcadas');
        const exigeAprovacao = exigeAprovacaoTodosAprovadores(widgetConfigAlcada);
        
        for (let i = 1; i <= 4; i++) {
            const fieldValor = widget.getField('VRAPROVCMP' + i);
            
            if (exigeAprovacao) {
                setLabel(fieldValor, "APPROVAL_ORDER");
            } else {
                setLabel(fieldValor, "APPROVAL_VALUE");
            }
            
            toggleFieldsOfApprovalValueAndOrder(fieldValor, exigeAprovacao);
        }
    };
    
    this.setFieldsOnDefaultMultiplosAprovadores = function(widget) {
        for (let i = 1; i <= 4; i++) {
            let approver = widget.getField('NMOPERADOR' + i);
            let value = widget.getField('VRAPROVCMP' + i);
            let justify = widget.getField('IDEXIGJUSTIFICATIVA' + i);
            
            approver.dataSourceFilter = [{
                name: "CDOPERADOR",
                operator: "NOT LIKE",
                value: "1099%"
            }];
            approver.dataSource.reload();

            value.readOnly = true;
            value.validations = {};
            
            justify.readOnly = true;
            justify.validations = {};
        }
    };
    
    this.setMultiplosAprovadoresVisibility = function(widget, isVisible) {
        for (let i = 1; i <= 4; i++) {
            widget.getField('NMOPERADOR' + i).isVisible = isVisible;
            widget.getField('VRAPROVCMP' + i).isVisible = isVisible;
            widget.getField('IDEXIGJUSTIFICATIVA' + i).isVisible = isVisible;
        }
    };
    
    this.setDataSourceFilterMultiplosAprovadores = function() {
        const widget = templateManager.container.getWidget('unidades');
        
        let dataSourceFilter = [{
            name: "CDOPERADOR",
            operator: "NOT LIKE",
            value: "1099%"
        }];
        
        for (let i = 1; i <= 4; i++) {
            let value = widget.getField('CDOPERADOR' + i).value();
            
            if (value != "") {
                dataSourceFilter.push({
                    name: "CDOPERADOR",
                    operator: "!=",
                    value: value
                });
            }
        }
        
        for (let i = 1; i <= 4; i++) {
            widget.getField('NMOPERADOR' + i).dataSourceFilter = dataSourceFilter;
            widget.getField('NMOPERADOR' + i).dataSource.reload();
        }
    };
    
    this.validateValueFieldsMultiplosAprovadores = function(widgetUnidades) {
        let arrayValues = [];
        
        for (let i = 1; i <= 4; i++) {
            let value = widgetUnidades.getField("VRAPROVCMP" + i).value();
            
            if (value != '' && value != null) {
                arrayValues.push(value);
            }
        }
        
        const uniqueValues = [...new Set(arrayValues)]; 
        
        return uniqueValues.length == arrayValues.length ? true : false;
    };
     
    this.setPropertiesDependentFieldsMultiplosAprovadores = function(rowNum) {
        const widget = templateManager.container.getWidget("unidades");
        const approver = widget.getField("NMOPERADOR" + rowNum);
        const value = widget.getField("VRAPROVCMP" + rowNum);
        const justify = widget.getField("IDEXIGJUSTIFICATIVA" + rowNum);
        const readOnly = (approver.value() == "");
        
        value.clearValue();
        justify.clearValue();
        
        value.readOnly = readOnly;
        justify.readOnly = readOnly;
        
        if (readOnly) {
            value.validations = {};
            justify.validations = {};
        } else {
            value.validations = {
                required: true
            };
            
            justify.validations = {
                required: true
            };
        }
    };
    
    this.saveMultiplosAprovadoresRowOnForm = function(widget, route) {
        removeActionsFromRow(widget);
        const currentRow = widget.currentRow;
        const rows = this.makeRowsToSaveMultiplosAprovadores(currentRow);

        configuracao_AlcadasService.requestRow(rows, route).then(
            function(response) {
                afterSaveSwipe(widget, getMessageFromResponse(response));
            }
        ).catch(
            function(error) {
                getCleanErrorMessage(error);
            }
        );
    };
    
    this.makeRowsToSaveMultiplosAprovadores = function(currentRow) {
        let rows = [];
        
        for (let index in currentRow.CDFILIAL) {
            for (let i = 1; i <= 4; i++) {
                if (currentRow['CDOPERADOR' + i] != '') {
                    rows.push({
                        __is_new: true,
                        NRCONFIGALCADA: currentRow.NRCONFIGALCADA,
                        CDFILIAL: currentRow.CDFILIAL[index],
                        CDOPERADOR: currentRow['CDOPERADOR' + i],
                        NMOPERADOR: currentRow['NMOPERADOR' + i],
                        VRAPROVCMP: currentRow['VRAPROVCMP' + i],
                        IDEXIGJUSTIFICATIVA: currentRow['IDEXIGJUSTIFICATIVA' + i]
                    });
                }
            }
        }
        
        return rows;
    };
    
    this.saveMultiplosAprovadoresAcessoSupply = function(widget, route) {
        const currentRow = widget.currentRow;
        const rows = this.makeRowsToSaveAcessoSupply(currentRow);
        
        configuracao_AlcadasService.requestRow(rows, route).then(
            function(response) {
                afterSaveSwipe(widget, getMessageFromResponse(response));
            }
        ).catch(
            function(error) {
                getCleanErrorMessage(error);
            }
        );
    };
    
    this.makeRowsToSaveAcessoSupply = function(currentRow) {
        let rows = [];
        
        for (let i = 1; i <= 4; i++) {
            let padrao = 'S';
            
            for (let index in currentRow.CDFILIAL) {
                let cdOperador = currentRow['CDOPERADOR' + i];
                
                if (cdOperador != '') {
                    rows.push({
                        CDOPERADOR: cdOperador,
                        CDFILIAL: currentRow.CDFILIAL[index],
                        IDPERFILACES: 'GO',
                        NRORG: getLocalVar('NRORG'),
                        IDATIVO: 'S',
                        IDFILIALPADRAO: padrao
                    });
                    
                    padrao = 'N';
                }
            }
        }

        return rows;
    };
    
    this.saveAprovadorAcessoSupply = function(widgetAprovadores, route) {
        const row = this.makeRowToSaveAcessoSupply(widgetAprovadores);
        
        configuracao_AlcadasService.requestRow(row, route).then(
            function(response) {
                afterSaveSwipe(widgetAprovadores, getMessageFromResponse(response));
            }
        ).catch(
            function(error) {
                getCleanErrorMessage(error);
            }
        );
    };
    
    this.makeRowToSaveAcessoSupply = function(widgetAprovadores) {
        const currentRowAprovador = widgetAprovadores.currentRow;
        const padrao = widgetAprovadores.dataSource.data.length > 1 ? 'N' : 'S';
        
        let row = [{
            CDOPERADOR: currentRowAprovador.CDOPERADOR,
            CDFILIAL: currentRowAprovador.CDFILIAL,
            IDPERFILACES: 'GO',
            NRORG: getLocalVar('NRORG'),
            IDATIVO: 'S',
            IDFILIALPADRAO: padrao
        }];
        
        return row;
    };
    
    /******************************************************
     *              UPLOAD DE PLANILHA (GR)               *
     ******************************************************/
    
    this.openUploadSpreadsheetPopUp = function() {
        const popup = templateManager.container.getWidget("importSpreadsheet");
        ScreenService.openPopup(popup);
    };
    
    this.confirmImport = function(widget) {
        if(widget.isValid()) {
            configuracao_AlcadasService.requestRow(widget.currentRow, 'GR_importSpreadsheet').then(function(callback) {
                const widgetResumo = templateManager.container.getWidget("configuracao_das_alcadas");
                const errors = Util.getNestedProperty(callback, ["dataset", "errors"]);
                
                if (Array.isArray(errors)) {
                    const message = self.buildErrorMessage(errors);
                    General.showErrorMessages(widgetResumo, message, ScreenService.i18n("Log de Erros:"));
                } else {
                    widgetResumo.reload().then(function() {
                        ScreenService.showMessage("Planilha importada com sucesso!", "success").then(function () {
                            ScreenService.closePopup(true);
                            widgetResumo.activate();
                        });
                    });
                }
            }).catch(self.handleSheetErrors);
        }
    };
    
    this.buildErrorMessage = function(errors) {
        let messages = [];
        
        for (let error of errors) {
            messages.push(error.DSERRO);
        }
        
        return messages;
    };
    
    this.handleSheetErrors = function (callback) {
        const errorMessages = Util.getNestedProperty(callback, ["data"]);

        if (Object.keys(errorMessages).length !== 0 && errorMessages.constructor === Object) {
            const detailedErrors = Util.getNestedProperty(callback, ["data", "dataset", "errors"]);
            const widgetResumo = templateManager.container.getWidget("configuracao_das_alcadas");
            
            if (Array.isArray(detailedErrors)) {
                widgetResumo.reload().then(function() {
                    let title = "Erros na planilha";
                    
                    General.showErrorMessages(widgetResumo, detailedErrors, ScreenService.i18n(title));
                });
            } else {
                widgetResumo.reload().then(function() {
                    ScreenService.showMessage("Erros na planilha => " + self.getErrorMessageTranslate(errorMessages.error), "error");
                });
            }
        } else {
            let message = "Tempo limite de conexão excedido.";
            
            if (callback.data && callback.data.error) {
                if (callback.data.error.toString().indexOf("Uninitialized string offset:") != -1) {
                    message = "Erro ao carregar planilha. Certifique-se de que usou o modelo de planilha para importação.";
                } else if (callback.data.error.toString().indexOf("failed to open stream: Permission denied") != -1) {
                    message = "Não foi possível carregar a planilha no servidor: Permissão negada. Por favor, contate um administrador do sistema.";
                }
            }
            
            ScreenService.showMessage(message, "error");
        } 
    };
    
    this.getErrorMessageTranslate = function(message) {
        const errorMessages = {
            "emptySheet": "Planilha Vazia.",
            "invalidSheetColumnsLabels": "Nomes das Colunas Inválidos."
        };
        
        return errorMessages[message];
    };
    
    /******************************************************
     * CONFIGURAÇÃO DAS ALÇADAS: UNIDADES                 *
     ******************************************************/
    
    this.onEnterUnidades = function() {
        const widgetUnidades = templateManager.container.getWidget('unidades');
        this.toggleVisibilityMultiplosAprovadores(widgetUnidades);
        
        toggleVisibilityConsistirValorAprovacao(widgetUnidades);
        setFilterOnWidgetUnidades(widgetUnidades);
    };
    
    function toggleVisibilityConsistirValorAprovacao(widgetUnidades) {
        const tipoAlcada = widgetUnidades.container.getWidget('configuracao_das_alcadas').currentRow.IDTPALCADA;
        const isVisible = (tipoAlcada == 'ORCAMENTO');
        
        widgetUnidades.getField('IDCONSISTVRAPROV').isVisible = isVisible;
        // Só é possível editar a unidade caso o campo acima esteja visível.
        widgetUnidades.getAction('edit').isVisible = isVisible;
    }
    
    this.newRowUnidades = function() {
        const widget = templateManager.container.getWidget('unidades');
        
        if (widget.isTypeCatalogo && widget.isImpGestUnidOff && widget.isGR) {
            this.changeValueFieldTypeMultiplosAprovadores(widget);
            this.setFieldsOnDefaultMultiplosAprovadores(widget);
            this.bindEventsMultiplosAprovadores(widget);
        }
        
        this.widgetNewRow(widget);
        toggleFieldUnidade(widget);
    };
    
    function verificaUnidadeRepetida(widgetUnidades){
        let unidade = 1;
        widgetUnidades.dataSource.data.forEach(function(data) {
            widgetUnidades.currentRow.CDFILIAL.forEach(function(CDFILIAL){
                if (data.CDFILIAL == CDFILIAL) {
                    unidade++;
                }
            });
        });
        return unidade <= 1 ? false:true;
    }
    
    this.saveFilialAlcada = function() {
        const widgetUnidadesAlcada = templateManager.container.getWidget('unidades');
        
        if (widgetUnidadesAlcada.isValid()) {
            if(verificaUnidadeRepetida(widgetUnidadesAlcada)){
                return MessageService.showMessageByCode("wkf#UNIDADE_CADASTRADA", {}, {
                    type: 'INFORMATION'
                });
            }
            if(widgetUnidadesAlcada.isTypeCatalogo && widgetUnidadesAlcada.isImpGestUnidOff && widgetUnidadesAlcada.isGR) {
                const validaValoresRepetidos = self.validateValueFieldsMultiplosAprovadores(widgetUnidadesAlcada);
                
                if (!validaValoresRepetidos) {
                    return MessageService.showMessageByCode("wkf#OPERATION_BLOCKED_VALUE_ORDER_PENDING_APPROVALS", {}, {
                        type: 'INFORMATION'
                    });
                }
    
                saveRowOnForm(widgetUnidadesAlcada, 'filial_alcada/save');
                this.saveMultiplosAprovadoresRowOnForm(widgetUnidadesAlcada, 'aprovador_multiplo/save');
                this.saveMultiplosAprovadoresAcessoSupply(widgetUnidadesAlcada, 'acesso_supply/save_multiplo');
            } else {
                saveRowOnForm(widgetUnidadesAlcada, 'filial_alcada/save');
            }
        }
    };
    
    this.editFilialAlcada = function(widget) {
        if (possuiAprovacoesPendentes(widget)) {
            return MessageService.showMessageByCode("wkf#OPERATION_BLOCKED_UNIT_PENDING_APPROVALS", {}, {
                type: 'INFORMATION'
            });
        }
        
        this.widgetEditOnMasterDetail(widget);
        toggleFieldUnidade(widget);
    };
    
    this.deleteFilialAlcada = function() {
        const widgetUnidadesAlcada = templateManager.container.getWidget('unidades');
        
        if (possuiAprovacoesPendentes(widgetUnidadesAlcada)) {
            MessageService.showMessageByCode("wkf#OPERATION_BLOCKED_UNIT_PENDING_APPROVALS", {}, {
                type: 'INFORMATION'
            });
        } else {
            deleteRowOnMasterDetail(widgetUnidadesAlcada, 'filial_alcada/remove');
        }
    };
    
    function toggleFieldUnidade(widget){
        const unidade = widget.getField('NMFILIAL');
        const unidadeSelect = widget.getField('CDFILIAL');
        const unidadeGrid = widget.getField('CDFILIAL_GRID');
        
        if (widget.currentRow.__is_new) {
            unidade.showOnForm = false;
            unidadeGrid.showOnForm = false;
            
            unidadeSelect.showOnForm = true;
            unidadeSelect.validations = {
                required: true,
            };
        } else {
            unidade.showOnForm = true;
            unidadeGrid.showOnForm = true;
            
            unidadeSelect.showOnForm = false;
            unidadeSelect.validations = {};
        } 
    }
     
    /******************************************************
     * CONFIGURAÇÃO DAS ALÇADAS: APROVADORES              *
     ******************************************************/
    
    this.onEnterAprovadores = function() {
        const widgetConfigAlcada = templateManager.container.getWidget('configuracao_das_alcadas');
        const tipoAprovacao = widgetConfigAlcada.currentRow.IDTIPOVRAPROV;
        const widgetUnidades = templateManager.container.getWidget('unidades');
        const widgetAprovadores = templateManager.container.getWidget('aprovadores');
        const fieldValor = widgetAprovadores.getField('VRAPROVCMP');
        const exigeAprovacao = exigeAprovacaoTodosAprovadores(widgetConfigAlcada);
        
        this.setFilterOnWidgetAprovadores();
        
        if (exigeAprovacao) {
            setLabel(fieldValor, "APPROVAL_ORDER");
        } else {
            setLabel(fieldValor, "APPROVAL_VALUE");
        }
        
        toggleFieldsOfApprovalValueAndOrder(fieldValor, exigeAprovacao, false, tipoAprovacao);
        
        if (widgetUnidades.isTypeCatalogo && widgetUnidades.isGR) {
            widgetAprovadores.showCheckbox = false;
        } else {
            widgetAprovadores.showCheckbox = true;
        }
    };
    
    this.newAprovador = function() {
        const widgetAprovadores = templateManager.container.getWidget('aprovadores');
        let fieldAprovador = widgetAprovadores.getField('NMOPERADOR');
        fieldAprovador.setValue("");
        fieldAprovador.widget.fields[1]._lastItemSearch = "";
        this.widgetNewRow(widgetAprovadores);
        this.setFilterOnSelectAprovadores();
        fieldAprovador.reload();
    };
    
    function existeValorIgual(fieldAtual) {
        const widgetConfigAlcada = fieldAtual.getParent();
        const valoresExistentes = widgetConfigAlcada.fields
                                            .filter(field => nameOfValueFields.includes(field.name) && field.name !== fieldAtual.name)
                                            .map(field => field.value());
        
        if (valoresExistentes.includes(getValueFromString(fieldAtual.value()))) {
            return true;
        }
        
        return false;
    }
    
    this.onChangeFieldValor = function(fieldValorAtual) {
        if (existeValorIgual(fieldValorAtual)) {
            MessageService.showMessageByCode("wkf#VALUE_ORDER_ALREADY_CHOSEN", {}, {
                type: 'INFORMATION'
            });
            fieldValorAtual.clearValue();
        }
    };
    
    this.convertCurrencyToNumber = function(currency) {
        if (typeof currency == "string") {
            if (currency.indexOf(',') != -1) {
                return Number(currency.replace(".", "").replace(",", ".").replace(/[R$,]+/g,""));
            } else {
                return Number(currency);
            }
        } else {
            return currency;
        }
    };
    
    function existeValorAprovacaoIgual(widgetAprovadores) {
        const aprovadorAtual = widgetAprovadores.currentRow;
        const valoresExistentes = widgetAprovadores.dataSource.data.filter(function(row) {
            if (self.convertCurrencyToNumber(aprovadorAtual.VRAPROVCMP) == self.convertCurrencyToNumber(row.VRAPROVCMP) && 
                aprovadorAtual.CDOPERADOR != row.CDOPERADOR) {
                return row;
            }
        });
        
        if (valoresExistentes.length > 0) {
            return true;
        }
        
        return false;
    }
    
    function verificaAprovadorRepetido(widgetAprovadores){
        let aprovador = 0;
        widgetAprovadores.dataSource.data.forEach(function(data) {
            if (data.CDOPERADOR == widgetAprovadores.currentRow.CDOPERADOR && data.NMOPERADOR == widgetAprovadores.currentRow.NMOPERADOR) {
                aprovador++;
            }
        });
        return aprovador <= 1 ? false:true;
    }
    
    this.saveAprovador = function() {
        const widgetAprovadores = templateManager.container.getWidget('aprovadores');
        const widgetUnidades = templateManager.container.getWidget('unidades');
        
        if (widgetAprovadores.isValid()) {
            if (existeValorAprovacaoIgual(widgetAprovadores)) {
                return MessageService.showMessageByCode("wkf#VALUE_ORDER_ALREADY_REGISTERED", {}, {
                    type: 'INFORMATION'
                });
            }
            if(verificaAprovadorRepetido(widgetAprovadores)){
                return MessageService.showMessageByCode("wkf#APROVADOR_CADASTRADO", {}, {
                    type: 'INFORMATION'
                });
            }
            
            saveRowOnForm(widgetAprovadores, 'aprovador/save');
            if (widgetUnidades.isTypeCatalogo) {
                this.saveAprovadorAcessoSupply(widgetAprovadores, 'acesso_supply/save');
            }
        }
    };
    
    this.deleteAprovador = function() {
        const widgetAprovadores = templateManager.container.getWidget('aprovadores');
        
        deleteRowOnMasterDetail(widgetAprovadores, 'aprovador/remove');
    };
    
    function getValorAprovacao(aprovador) {
        return getValueFromString(aprovador.VRAPROVCMP);
    }
     
    /******************************************************
     * CONFIGURAÇÃO DAS ALÇADAS: APROVADORES RELACIONADOS *
     ******************************************************/
    
    this.newAprovadorRelacionado = function() {
        const widgetAprovadoresRelacionados = templateManager.container.getWidget('aprovadores_relacionados');
        let fieldAprovador = widgetAprovadoresRelacionados.getField('NMOPERADOR');
        fieldAprovador.setValue("");
        fieldAprovador.widget.fields[1]._lastItemSearch = "";
        fieldAprovador.reload();
        this.widgetNewRow(widgetAprovadoresRelacionados);
        this.setFilterOnSelectAprovadoresRelacionados();
    };
    
    this.saveAprovadorRelacionado = function() {
        const widgetAprovadoresRelacionados = templateManager.container.getWidget('aprovadores_relacionados');
        
        if (widgetAprovadoresRelacionados.isValid()) {
            /**
             * Inverte os operadores devido a uma limitação do outData do zh-select-autocomplete.
             * Até o dia 24/07/2019 não havia uma solução pelo próprio componente.
             * 
             * CDOPERADOR - Deve ser o código do Aprovador da tela "Aprovador"
             * CDOPERRELACAP - Deve sero código do Aprovador Relacionado
             */
            widgetAprovadoresRelacionados.currentRow.CDOPERRELACAP = widgetAprovadoresRelacionados.currentRow.CDOPERADOR;
            widgetAprovadoresRelacionados.currentRow.CDOPERADOR = widgetAprovadoresRelacionados.getParent().currentRow.CDOPERADOR;
            
            saveRowOnForm(widgetAprovadoresRelacionados, 'aprovador_relacionado/save');
        }
    };
    
    this.deleteAprovadorRelacionado = function() {
        const widgetAprovadoresRelacionados = templateManager.container.getWidget('aprovadores_relacionados');
        
        deleteRowOnMasterDetail(widgetAprovadoresRelacionados, 'aprovador_relacionado/remove');
    };


    /****************************
     *         FILTROS          *
     ****************************/

    function setFilterOnWidgetUnidades(widgetUnidades) {
        widgetUnidades.dataSourceFilter = [
            {
                name: 'NRCONFIGALCADA',
                operator: '=',
                value: widgetUnidades.getParent().currentRow.NRCONFIGALCADA
            }
        ];
        
        widgetUnidades.reload();
    }

    this.setFilterOnWidgetAprovadores = function() {
        const widgetAprovadores = templateManager.container.getWidget('aprovadores');

        widgetAprovadores.dataSourceFilter = [
            {
                name: 'NRCONFIGALCADA',
                operator: '=',
                value: widgetAprovadores.getParent().currentRow.NRCONFIGALCADA
            },
            {
                name: 'CDFILIAL',
                operator: '=',
                value: widgetAprovadores.getParent().currentRow.CDFILIAL
            },
            {
                operator: 'ORDER_BY',
                name: [
                    {
                        name: 'VRAPROVCMP',
                        direction: 'DESC'
                    }
                ]
            }
        ];

        widgetAprovadores.reload();
    };

    this.setFilterOnWidgetAprovadoresRelacionados = function() {
        const widgetAprovadoresRelacionados = templateManager.container.getWidget('aprovadores_relacionados');
        const widgetAprovadores = widgetAprovadoresRelacionados.getParent();

        widgetAprovadoresRelacionados.dataSourceFilter = [
            {
                name: 'NRCONFIGALCADA',
                operator: '=',
                value: widgetAprovadores.currentRow.NRCONFIGALCADA
            },
            {
                name: 'CDFILIAL',
                operator: '=',
                value: widgetAprovadores.currentRow.CDFILIAL
            },
            {
                name: 'CDOPERADOR',
                operator: '=',
                value: widgetAprovadores.currentRow.CDOPERADOR
            }
        ];

        widgetAprovadoresRelacionados.reload();
    };

    function isPersisted(item) {
        return !item.__is_new;
    }
    
    this.setFilterOnSelectUnidades = function() {
        const widgetUnidades = templateManager.container.getWidget('unidades');
        
        if (widgetUnidades.inEditMode()) {
            const fieldUnidades = widgetUnidades.getField('CDFILIAL');
            const unidadesExistentes = widgetUnidades.dataSource.data.filter(isPersisted);
            const newDataSourceFilter = [
                {
                    name: 'P_CDOPERADOR',
                    operator: '=',
                    value: getLocalVar('CDOPERADOR')
                },
            ];
        
            unidadesExistentes.forEach(function(value, index) {
                newDataSourceFilter.push(
                    {
                        name: 'CDFILIAL',
                        operator: '<>',
                        value: value.CDFILIAL
                    }
                );
            });
            
            fieldUnidades.dataSourceFilter = newDataSourceFilter;
        }
    };

    this.setFilterOnSelectAprovadores = function() {
        const widgetAprovadores     = templateManager.container.getWidget('aprovadores');
        const fieldAprovadores      = widgetAprovadores.getField('NMOPERADOR');
        const aprovadoresExistentes = widgetAprovadores.dataSource.data.filter(isPersisted);
        const newDataSourceFilter   = [];

        aprovadoresExistentes.forEach(function(value, index) {
            newDataSourceFilter.push({
                name: 'CDOPERADOR',
                operator: '<>',
                value: value.CDOPERADOR
            });
        });

        fieldAprovadores.dataSourceFilter = newDataSourceFilter;
    };

    this.setFilterOnSelectAprovadoresRelacionados = function() {
        const widgetAprovadores                 = templateManager.container.getWidget('aprovadores');
        const aprovadoresExistentes             = widgetAprovadores.dataSource.data;
        const widgetAprovadoresRelacionados     = templateManager.container.getWidget('aprovadores_relacionados');
        const aprovadoresRelacionadosExistentes = widgetAprovadoresRelacionados.dataSource.data;
        const fieldNomeAprovadorRelacionado     = widgetAprovadoresRelacionados.getField('NMOPERADOR');

        fieldNomeAprovadorRelacionado.dataSourceFilter = [];

        aprovadoresRelacionadosExistentes.forEach(function(value, index, array) {
            if (index < array.length - 1) {
                fieldNomeAprovadorRelacionado.dataSourceFilter.push({
                    "name": "CDOPERADOR",
                    "operator": "<>",
                    "value": value.CDOPERRELACAP
                });
            }
        });

        aprovadoresExistentes.forEach(function(value, index, array) {
            if (index < array.length) {
                fieldNomeAprovadorRelacionado.dataSourceFilter.push({
                    "name": "CDOPERADOR",
                    "operator": "<>",
                    "value": value.CDOPERADOR
                });
            }
        });

        fieldNomeAprovadorRelacionado.reload();
    };
    
    this.setFilterOnSelectAprovadoresSubstitutos = function() {
        const widgetAprovadores     = templateManager.container.getWidget('aprovadores');
        const widgetAprovadoresSubs = templateManager.container.getWidget('popUpSubstituirAprovador');
        const fieldAprovadoresSubs  = widgetAprovadoresSubs.getField('NMOPERADOR_popup');
        const aprovadoresExistentes = widgetAprovadores.dataSource.data.filter(isPersisted);
        const newDataSourceFilter   = [];

        aprovadoresExistentes.forEach(function(value, index) {
            newDataSourceFilter.push({
                name: 'CDOPERADOR',
                operator: '<>',
                value: value.CDOPERADOR
            });
        });

        fieldAprovadoresSubs.dataSourceFilter = newDataSourceFilter;
    };
    
    function exibeLog(message) {
        const widgetOrigem = templateManager.container.getActiveWidget();
        const mensagemFormatada = [];
        const separador = '=======================================';
        const montaCabecalho = function(informacaoCabecalho) {
            mensagemFormatada.push(separador);
            mensagemFormatada.push(informacaoCabecalho);
            mensagemFormatada.push(separador);
        };
        const isPrimeiraMensagem = function(index) {
            return index === 0;
        };
        const isUltimaMensagem = function(index, arrayRetorno) {
            return index === arrayRetorno.length - 1;
        };
        
        message.forEach(retorno => {
            retorno.split('|').forEach((data, index, arrayRetorno) => {
                if (isPrimeiraMensagem(index)) {
                    montaCabecalho(data);
                } else if(isUltimaMensagem(index, arrayRetorno)) {
                    mensagemFormatada.push(data + '\n');
                } else {
                    mensagemFormatada.push(data);
                }
            });
        });
        
        
        General.showErrorMessages(widgetOrigem, mensagemFormatada, ScreenService.i18n('Log'));
    }
    this.openSubstituirAprovador = function(){
		const widgetPopUp = templateManager.container.getWidget('popUpSubstituirAprovador');
		let fieldPopUp = widgetPopUp.getField('NMOPERADOR_popup');
		fieldPopUp.setValue("");
        fieldPopUp.widget.fields[1]._lastItemSearch = "";
		ScreenService.openPopup(widgetPopUp);
		this.setFilterOnSelectAprovadoresSubstitutos();
        fieldPopUp.reload();
    };
    
    this.closeSubstituirAprovador = function(widget){
        ScreenService.closePopup();
    };
    
    function verificaAprovadorSubstituto(widgetAprovadores, widgetPopUp){
        let aprovador = 1;
        widgetAprovadores.dataSource.data.forEach(function(data) {
            if (data.CDOPERADOR == widgetPopUp.currentRow.CDOPERADOR_popup && data.NMOPERADOR == widgetPopUp.currentRow.NMOPERADOR_popup) {
                aprovador++;
            }
        });
        return aprovador <= 1 ? false:true;
    }
    
    this.substituirAprovador = function (widget){
        if(widget.isValid()){
    		let widgetUnidades = templateManager.container.getWidget('unidades');
    		let widgetAprovadores  = templateManager.container.getWidget('aprovadores');
            let widgetPopUp = templateManager.container.getWidget('popUpSubstituirAprovador');
            let row = {
                "CDFILIAL": widgetUnidades.currentRow.CDFILIAL,
                "CDOPERADOR": widget.currentRow.CDOPERADOR_popup,
                "NRCONFIGALCADA": widgetUnidades.currentRow.NRCONFIGALCADA,
                "CDOPERADOR_OLD": widgetAprovadores.currentRow.CDOPERADOR,
                "IDAPROVADO": "PENDENTE"
            };
            
            if(verificaAprovadorSubstituto(widgetAprovadores, widgetPopUp, 1)){
                return MessageService.showMessageByCode("wkf#APROVADOR_CADASTRADO", {}, {
                    type: 'INFORMATION'
                });
            }
            
            configuracao_AlcadasService.atualizarAprovador(row).then(function() {
                ScreenService.closeSwipe();
                ScreenService.closePopup();
                ScreenService.closePopup();
                widgetPopUp.getField('NMOPERADOR_popup').setValue('');
                widgetAprovadores.reload();
            }).catch(
                function(error) {
                    getCleanErrorMessage(error);
                }
            );
        }  
    };
    
    this.enviarQrcode = function() {
        let widgetAtiva = templateManager.container.getWidget('aprovadores');
        let params = [{ "name": "IDAPLICATIVO", "operator": "=", "value": "APPROVE" }];
        
        configuracao_AlcadasService.retornaDnsApp(params).then(function(appconfig) {
            appconfig = appconfig.dataset.appconfig;
            
            if(appconfig.length && appconfig[0].DSDNS) {
                let dns = appconfig[0].DSDNS + ';' + getLocalVar('NRORG');
                let qrCodeContainer = document.createElement('span');
                
                let nrconfigalcada = retornaParametroEnvioQrcode(widgetAtiva, 'NRCONFIGALCADA');
                let cdfilial = retornaParametroEnvioQrcode(widgetAtiva, 'CDFILIAL');
                let cdoperador = retornaParametroEnvioQrcode(widgetAtiva, 'CDOPERADOR', true);
                let cdoperrelacap = retornaParametroEnvioQrcode(widgetAtiva, 'CDOPERRELACAP', true);
                
                new QRCode(qrCodeContainer, dns);
    
                enviaEmailQrcode(dns, nrconfigalcada, cdfilial, cdoperador, cdoperrelacap);
                
                widgetAtiva.unSelectAll();
                
            } else {
                MessageService.showMessageByCode("wkf#THERE_ARE_NO_DNS", {}, {
                    type: 'INFORMATION'
                });
            }
            
        });
        
    };
    
    function enviaEmailQrcode(dns, nrconfigalcada, cdfilial, cdoperador, cdoperrelacap) {
        const arrayMessage = [
            'MESSAGE_APPROVE_1',
            'MESSAGE_APPROVE_2',
            'MESSAGE_APPROVE_3',
            'MESSAGE_APPROVE_4',
            'MESSAGE_APPROVE_5'
        ];
        let promises = [];
        let messages = '';
        let paragrafoDuplo = 0;
        
        MessageService.getMessageByCode('wkf#ALL_RIGHTS').then(function(allRights) {
            MessageService.getMessageByCode('wkf#SUBJECT_APPROVE').then(function(subject) {
                arrayMessage.forEach(function(key){
                    var messagePromise = MessageService.getMessageByCode('wkf#'.concat(key)).then(function(message) {
                        messages += message + ' </br> ';
                        paragrafoDuplo += 1;
                        if(paragrafoDuplo > 2 && paragrafoDuplo < 5) {
                            messages += ' </br> ';
                        }
                    });
                    promises.push(messagePromise);    
                });
                
                ZHPromise.all(promises).then(function() {
                    setTimeout(function() {
                        let params = {
                            'qrCode': {
                                'data': getLocalVar('qrcode_uri'),
                                'url': dns
                            },
                            'NRCONFIGALCADA': nrconfigalcada,
                            'CDFILIAL': cdfilial,
                            'CDOPERADOR': cdoperador,
                            'CDOPERRELACAP': cdoperrelacap,
                            'SUBJECT': subject,
                            'MESSAGE': messages,
                            'ALL_RIGHTS': allRights
                        };
    
                        configuracao_AlcadasService.enviaEmailQrcode(params).then(function() {
                            MessageService.successNotificationByCode("wkf#SEND_SUCCESS_INVITATIONS");
                        }).catch(function(erro) {
                            MessageService.showMessageByCode("wkf#TRY_AGAIN", {}, {
                                type: 'INFORMATION'
                            });
                        });
    
                    }, 350);
                });
            });
        });
    }
}

Configuration(function(ContextRegister) {
    ContextRegister.register('configuracao_AlcadasController', configuracao_AlcadasController);
});

